/// \file mil1553ud_ioctl.h
/// \brief инфраструктура ioctl-команд
#ifndef MIL1553UD_CMD_H
#define MIL1553UD_CMD_H


#ifdef __KERNEL__
#include <linux/types.h>
#else /* USER */
#include <stdint.h>
#include <stdbool.h>
#endif

/// \remark константы для управления прерываниями
/// 0 - не меняется значение, 1 - включить прерывание, 2 - выключить прерывание
/// \brief не изменять бит прерывания
#define INTERRUPT_MAN_NO_CHANGE         0
/// \brief включить бит прерывания
#define INTERRUPT_MAN_ON                1
/// \brief выключить бит прерывания
#define INTERRUPT_MAN_OFF               2

/// \remark константы режимов работы устройства
/// \brief адресуемый монитор шины
#define MIL_MODE_MONITOR_ADRR           0x0
/// \brief контроллер шины
#define MIL_MODE_BUS_CONTR              0x1
/// \brief оконечное устройство
#define MIL_MODE_TERMINAL_DEV           0x2
/// \brief НЕадресуемый монитор шины
#define MIL_MODE_MONITOR                0x3

/// \remark константы выбора шины канала
/// \brief включить шину А
#define MIL_BUS_A_EN                    0x1
/// \brief включить шину В
#define MIL_BUS_B_EN                    0x2

/// \remark константы вкл/выкл канала
/// \brief включить канал в работу
#define MIL_DEV_CHANNEL_ON              0x1
/// \brief выключить канал
#define MIL_DEV_CHANNEL_OFF             0x0

/// \remark константы для режима ОУ
/// \brief режим - программный
#define MIL_RT_MODE_PROG                0x0
/// \brief режим - аппаратный
#define MIL_RT_MODE_HRDW                0x1

/// \remark команды управления буферами cmd_buf
/// \brief буфер данных RTF_BUF0 и RTF_BUF1 - выкл
#define MIL_RT_BUF_0_OFF_1_OFF          0x0
/// \brief буфер данных RTF_BUF0 - выкл и RTF_BUF1 - вкл
#define MIL_RT_BUF_0_OFF_1_ON           0x1
/// \brief буфер данных RTF_BUF0 - вкл и RTF_BUF1 - выкл
#define MIL_RT_BUF_0_ON_1_OFF           0x2
/// \brief буфер данных RTF_BUF0 - вкл и RTF_BUF1 - вкл
#define MIL_RT_BUF_0_ON_1_ON            0x3

/// \remark направление передачи direction
/// \brief направление - передача
#define MIL_RT_BUF_TRANSMIT             0x0
/// \brief направление - передача
#define MIL_RT_BUF_RECEIVE              0x1

/// \remark номер буфера num_buf
/// \brief отправной буфер данных RTF_BUF0
#define MIL_RT_BUF0                     0
/// \brief отправной буфер данных RTF_BUF1
#define MIL_RT_BUF1                     1

/// \remark константы для timer timeout RCVCK
#define MIL_T_RCVCK_17MKS               0x0
#define MIL_T_RCVCK_60MKS               0x1
#define MIL_T_RCVCK_85MKS               0x2
#define MIL_T_RCVCK_110MKS              0x3

/// \remark константы для timer timeout TRCK
#define MIL_T_TRCK_6MKS                 0x0
#define MIL_T_TRCK_8MKS                 0x1
#define MIL_T_TRCK_11MKS                0x2
#define MIL_T_TRCK_13MKS                0x3
#define MIL_T_TRCK_18MKS                0x4
#define MIL_T_TRCK_61MKS                0x5
#define MIL_T_TRCK_86MKS                0x6
#define MIL_T_TRCK_111MKS               0x7

/// \remark константы для timer timeout TCK
#define MIL_T_TCK_OFF                   0x0
#define MIL_T_TCK_1MKS                  0x1
#define MIL_T_TCK_2MKS                  0x2
#define MIL_T_TCK_4MKS                  0x3
#define MIL_T_TCK_8MKS                  0x4
#define MIL_T_TCK_16MKS                 0x5
#define MIL_T_TCK_32MKS                 0x6
#define MIL_T_TCK_64MKS                 0x7

/// \remark константы для команды записи/чтения BC RAM
#define MIL_BC_RAM_TYPE_INSTRUCTION     0x0
#define MIL_BC_RAM_TYPE_OPERATION       0x1
#define MIL_BC_RAM_TYPE_DATA            0x2

/// \remark номера каналов
/// \brief канал 1
#define CH_NUM_1            0
/// \brief канал 2
#define CH_NUM_2            1
/// \brief канал 3
#define CH_NUM_3            2
/// \brief канал 4
#define CH_NUM_4            3

/// \brief блок информации о прерывании
struct block_info {
    /// \brief значение таймера в момент прерывани
    unsigned int    free_timer_value;
    /// \brief маска прерываний (собственно по ней поймём какие в этот момент времени были прерывания)
    /// 0 - INT_HDAT, 1 - INT_QDAT, 2 - RT_INT_SADDR, 3 - RT_INT_MC_ERR,
    /// 4 - INT_BC, 5 - INT_FLASH, 6 - INT_DATA_CNT_EN,
    /// 7 - INT_TIMEOUT_ITVEN, 8 - INT_TIMEOUT_ABSEN
    unsigned int    interrupt_ch;
    /// \brief регистр HW_STAT_REG1
    unsigned int    hw_stat_reg1;
    /// \brief регистр HW_STAT_REG2
    unsigned int    hw_stat_reg2;
};

#define BLINF_INT_HDAT          0
#define BLINF_INT_QDAT          1
#define BLINF_RT_INT_SADDR      2
#define BLINF_RT_INT_MC_ERR     3
#define BLINF_INT_BC            4
#define BLINF_INT_FLASH         5
#define BLINT_INT_DATA_CNT_EN   6
#define BLINT_TIMEOUT_ITVEN     7
#define BLINT_TIMEOUT_ABSEN     8

#define SIZE_OF_PCI_LOC         13

///
/// \remark ТИПЫ ДАННЫХ АРГУМЕНТОВ IOCTL-команд
///

#define MIL1553_ONE_BLOCK_SIZE_128_BYTES        128

/// \brief контейнер блоков ДМА (на 256 блоков - максимум)
typedef struct {
    unsigned int currentBlocks;      ///< фактическое количество блоков ДМА (не более mallocedBlocks)
    unsigned int nextBlocks;         ///< кол-во блоков, готовых к чтению в целевом устройстве (кольцевом буфере ДМА девайса)
    unsigned int mallocedBlocks;     ///< выделенное место в data
    unsigned char* data;    ///< блоки ДМА в сыром виде (кратные 128 байтам)
} MIL1553_DMA_FIX_BLOCK;

/// \brief информация о локации на шине pci
typedef struct {
    ///
    /// \brief name инфо о локации
    /// например, 0000:00:02.0 -
    /// выделены домен (16 бит), шина (8 бит), устройство (5 бит) и функция (3 бита)
    ///
    char name[SIZE_OF_PCI_LOC];
} PCI_LOCATION;

#define SIZE_OF_ETH_LOC 25

/// \brief информация о локации ethernet
typedef struct {
    ///
    /// \brief name инфо о локации
    /// например, 192.168.1.23:9900 (ip-адрес и порт)
    ///
    char ip_address_and_port[SIZE_OF_ETH_LOC];
} ETH_LOCATION;

#define SIZE_OF_USB_LOC 128

/// \brief информация о локации USB
typedef struct {
    ///
    /// \brief name инфо о локации
    /// например, /dev/ttyACM0
    ///
    char path[SIZE_OF_USB_LOC];
} USB_LOCATION;

/// \brief информация о драйвере и устройстве
typedef struct {
    /// \brief идентификатор устройства
    unsigned int device_id;
    /// \brief вендор устройства
    unsigned int vendor_id;
    /// \brief тип устройства (кол-во каналов)
    unsigned int type;
    /// \brief ревизия устройства
    char revision;
    /// \brief имя символьного устройства
    char dev_name[30];
    /// \brief значение минора
    int minor;
    /// \brief номер прерывания
    int irq;
    /// \brief размер ДМА буфера
    long size_dma;
    /// \brief виртуальный адрес ДМА буфера
    void* addr_dma_virt;
    /// \brief адрес bar-пространства
    unsigned int pci_bars;
    /// \brief виртуальный адрес bar-пространства
    void* addr_bar_virt;
} VERSION;

/// \brief запись/чтение регистров
typedef struct {
    /// \brief адрес регистра (смещение в соотв. со спецификацией)
    unsigned long daddr;
    /// \brief значение регистра
    unsigned int data;
} SADDR_DATA;

/// \brief запись в регистр заданных бит
typedef struct {
    /// \brief адрес регистра (смещение в соотв. со спецификацией)
    unsigned long daddr;
    /// \brief значение регистра
    unsigned int data;
    /// \brief битовая маска
    /// 1 - бит записывается из data, 0 - бит остаётся неизменным
    unsigned int mask;
} SADDR_DATA_BIT_MASK;

/// \brief считываемый блок дма, кратный 128 байтам
typedef struct {
    /// \brief количество 128 байтных блоков
    unsigned int countBlocks;
    /// \brief длина data в байтах
    unsigned int length;
    /// \brief данные блоков в сыром виде
    unsigned char* data;
} DMA_READ_BLOCK;

/// \remark типы данных для работы с прервываниями

/// \brief управление маскированием перываний поканально
typedef struct {
    /// \brief состояние прерывания от контроллера flash
    unsigned char int_flash;
    /// \brief состояние прерывания от КШ
    unsigned char int_bc;
    /// \brief состояние прерывания от ОУ при приёме данных
    unsigned char int_rt_ren;
    /// \brief состояние прерывания от ОУ при отправке данных
    unsigned char int_rt_ten;
    /// \brief состояние прерывания при заполнении 1/18
    unsigned char int_qdat;
    /// \brief состояние прерывания при заполнении 1/2
    unsigned char int_hdat;
    /// \brief состояние прерывания счётчика данных контроллера MIL
    unsigned char int_data_cnt_en;
    /// \brief состояние прерывания интервального таймера MIL
    unsigned char int_timeout_itven;
    /// \brief состояние прерывания абсолютного таймера MIL
    unsigned char int_timeout_absen;
} INTERRUPT_MAN;

/// \brief управление маскированием прерываний по приёму/передачи данных по подадресам
typedef struct {
    /// \brief состояние прерывания (вкл - 1 / выкл - 2)
    unsigned char rt_int;
    /// \brief маскирование прерываний по подадресам 1й бит - 1й подадрес и т.д.
    unsigned int subaddress_mask;
} INTERRUPT_SUBADDRESS;

/// \remark типы данных для работы с режимами

/// \brief управление режимом работы ОУ по подадресам
typedef struct {
    /// \brief подадрес
    unsigned char subaddress;
    /// \brief режим
    unsigned int mode;
} RT_TR_MODE;

/// \brief управление буферами 1 и 2
typedef struct {
    /// \brief подадрес 1 - 30
    unsigned char subaddress;
    /// \brief команда упр. буфером
    unsigned int cmd_buf;
} RT_TR_MODE_BUF;

/// \brief управление подадресами в режиме ОУ
typedef struct {
    /// \brief направление (приём/передача)
    unsigned int direction;
    /// \brief маскирование (выбор) по подадресам 1й бит - 1й подадрес и т.д.
    /// 1 - бит изменяем, 0 - бит не изменяем
    unsigned int subaddress_mask;
    /// \brief маскирование (выбор) действий по подадресам 1й бит - 1й подадрес и т.д.
    /// 1 - вкл, 0 - выкл
    unsigned int action_mask;
} RT_BUF_MAN_EN;

/// \brief буфер ОУ на передачу подадреса
typedef struct {
    /// \brief номер буфера
    unsigned char num_buf;
    /// \brief подадрес (1-30)
    unsigned char subaddress;
    /// \brief подадрес (32 слова данных)
    unsigned short data_words[32];
} RT_TR_BUF_SUBADDR;

/// \brief управление таймаутами таймеров
typedef struct {
    /// \brief значение RT_TCK
    unsigned int tck;
    /// \brief значение RT_TRCK
    unsigned int trck;
    /// \brief значение RT_RCVCK
    unsigned int rcvck;
} TIMER_TIMEOUT;

/// \brief блок данных для записи в BC_RAM (область микропрограммы для КШ)
typedef struct {
    /// \brief тип инструкций
    /// INSTR, OPERATION, DATA
    unsigned int type;
    /// \brief смещение от начала буфера в 32-х разрядных словах
    /// INSTR - max 4095, OPERATION - max 4095, DATA - max 8191
    unsigned int shift;
    /// \brief размер поля dwords в 32-х разрядных словах
    unsigned int length;
    /// \brief данные (массив 32-х разрядных слов)
    unsigned int* dwords;
} BC_RAM_BLOCK;

/// \brief блок данных для чтения накопленных прерываний
typedef struct {
    /// \brief кол-во запришиваемых блоков
    unsigned int        count_blocks;
    /// \brief собственно указатель на массив блоков
    struct block_info*  blocks;
} INTERRUPT_BLOCK_BUFFER;

/// \brief информация о подадресе
typedef struct {
    /// \brief подадрес (1-30)
    unsigned char sa;
    /// \brief слова данных
    unsigned short words[32];
} SUBA_DATA_BLOCK;

/// \brief информация для генерации сигнала
typedef struct {
    /// \brief pid целевого процесса
    /// кому посылать
    int target_pid;
    /// \brief используемый сигнал
    int sig_number;
}
SIGNAL_INFO;

#define FW_BINARY_BLOCK_SIZE 512

/// \brief Структура блока обновления ПО lan1553ud
typedef struct {
    /// \brief указатель на часть прошивки
    void *binaryBlock;

    /// \brief индекс блока
    unsigned blockSize;

    /// \brief индекс блока
    unsigned blockOffset;

    /// \brief общий размер прошивки
    unsigned fileSize;
}
FW_BLOCK;


#define CONF_BLOCK_UPDATE_DESC_SIZE 64

/// \brief Структура блока обновления ПО lan1553ud
typedef struct {
    unsigned block_id; ///< [in] Индекс запрошенного раздела обновления
    bool is_protected; ///< [out] Разрешено ли обновление в этот раздел

    unsigned active_block_id; ///< [out] Индекс загрузочного раздела обновления

    uint8_t mac_addr[6]; ///< мак-адрес

    uint8_t is_static_ip_enabled; ///< если false, адрес будет взят по dhcp. Если true - из поля ниже
    uint8_t ipv4_addr[4]; ///< адрес интерфейса

    uint8_t is_netmask_set; ///< если false, шлюз не учитывается
    uint8_t ipv4_netmask[4]; ///< сетевая маска

    uint8_t is_gateway_set; ///< если false, шлюз не учитывается
    uint8_t ipv4_gateway[4]; ///< шлюз

    uint64_t mil_serial_num; ///< Уникальный идентификатор устройства в серии (для различения нескольких usb)

    uint16_t server_port; ///< порт для внутреннего ПО mil-eth-server

    char update_desc[ CONF_BLOCK_UPDATE_DESC_SIZE ]; ///< строка с описанием обновления (ascii латиница)
}
CONF_BLOCK;

/// Стадии обновления
enum UpdateStage
{
    STAGE_IDLE             = 0x1100, ///< Ожидание загрузки, обновление не производится
    STAGE_UPLOAD           = 0x0101, ///< Загрузка в память платы
    STAGE_ERASE            = 0x0002, ///< Очистка флеш-памяти под будущее обновление
    STAGE_FLASH            = 0x0003, ///< Прошивка в флеш-память платы
    STAGE_VERIFY           = 0x0004, ///< Верификация
    STAGE_SET_CONFIG       = 0x0005, ///< Установка настроек
    STAGE_SUCCESS          = 0x1106, ///< Успешное завершение
    STAGE_ERROR            = 0x1107, ///< Ошибка при обновлении

    STAGE_IS_IDLE_MASK     = 0x1000,
    STAGE_ALLOW_RESET_MASK = 0x0100,
};

/// Структура для запроса статуса обновления ПО lan1553ud
typedef struct {
    int stage; ///< Этап обновления
    int percentComplete; ///< процент завершения
}
FW_STATUS;

/// \remark КОДЫ ОШИБОК IOCTL-запросов

/// \brief нет ошибок
#define GOOD                            0
/// \brief ошибка
#define ERR_UNKNOWN                     -1000
/// \brief ошибка адреса регистра
#define ERR_BAD_ADDRESS                 -1001
/// \brief ошибка номера канала
/// внутрення ошибка в драйвере
#define ERR_BAD_CHANNEL                 -1002
/// \brief ошибка копирования данных в юзерспейс
#define ERR_COPY_TO_USER                -1003
/// \brief ошибка аргумента - mode
#define ERR_BAD_ARG_MODE                -1004
/// \brief ошибка аргумента - subaddress
#define ERR_BAD_ARG_SA                  -1005
/// \brief ошибка аргумента - direction
#define ERR_BAD_ARG_DIRECTION           -1006
/// \brief ошибка аргумента - tck
#define ERR_BAD_ARG_TCK                 -1007
/// \brief ошибка аргумента - trck
#define ERR_BAD_ARG_TRCK                -1008
/// \brief ошибка аргумента - rcvck
#define ERR_BAD_ARG_RCVCK               -1009
/// \brief ошибка аргумента - numbuf
#define ERR_BAD_ARG_NUMBUF              -1010
/// \brief ошибка аргумента - typr
#define ERR_BAD_ARG_TYPE                -1011
/// \brief ошибка выделения памяти
/// внутренняя ошибка драйвера
#define ERR_BAD_ALLOC_MEM               -1012
#define ERR_BAD_DMA_INDEX               -1013
#define ERR_BAD_DMA_NBLOCK              -1014


#endif // MIL1553UD_CMD_H
