#ifndef MIL1553UD_COMMON_H
#define MIL1553UD_COMMON_H

#include "mil1553ud_cmd.h"
#include "mil1553ud_regs.h"

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <time.h>
#include <errno.h>
#include <ctype.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef _WIN32
    #include <winsock2.h>
    #include <windows.h>
#else
    #include <sys/ioctl.h>  /* ioctl */
    #include <unistd.h>
    #include <strings.h>
    #include <sys/time.h>
    #include <dirent.h>
    #include <sys/param.h>
#endif

#include <string.h>

#ifdef __cplusplus
extern "C" {
#endif

/// \remark Старые функции имеют префикс extmil1553ud_
/// Новые функции имеют префикс extmil1553_

/// /// /// /// /// /// /// /// /// /// /// ///
/// \remark МАКРОБИБЛИОТЕКА MIL1553. ТИПЫ ДАННЫХ и ОБЪЯВЛЕНИЯ.
/// /// /// /// /// /// /// /// /// /// /// ///

/// \brief код значения - хорошо
#define GOOD_CODE                       0
/// \brief код значения - плохо
#define BAD_CODE                        -1

/// \brief длина имени канала
#define CHANNEL_NAME_LENGTH             30

#define MIL1553_OFF     0
#define MIL1553_ON      (~MIL1553_OFF)

///
/// \brief информация о канале
///
struct TMil1553Channel {
    char mNameChannel[CHANNEL_NAME_LENGTH];             ///< полное имя канала
    int  mFD;                                           ///< дескриптор файла
};

///
/// \brief The TMil1553Device struct информация о девайсе
///
struct TMil1553Device {
    struct TMil1553Channel  mDev;                       ///< канал, олицетворяющий весь девайс
    VERSION mInfo;                                      ///< инфо о девайсе
};

/// \brief размер блока дма mil1553
#define MIL1553_DMA_BLOCK_SIZE      128
///
/// \brief The TMIL1553_DMA_BLOCK struct блок дма mil1553
///
struct TMIL1553_DMA_BLOCK {
    unsigned char mData[MIL1553_DMA_BLOCK_SIZE];        ///< содержимое блока
};

/// \brief максимальный размер контейнера блоков дма mil1553
#define MIL1553_DMA_CONTAINER_BLOCKS_MAX_SIZE   128
///
/// \brief The MIL1553_DMA_CONTAINER_BLOCKS struct контейнер блоков дма mil1553
///
struct MIL1553_DMA_CONTAINER_BLOCKS {
    struct TMIL1553_DMA_BLOCK mBlocks[MIL1553_DMA_CONTAINER_BLOCKS_MAX_SIZE];   ///< блоки ДМА
    int mCount; ///< кол-во блоков
};

///
/// \brief The EMil1553BcTypeDataRam enum тип озу контроллера шины
///
enum EMil1553BcTypeDataRam {
    Mil1553BcTypeInstructions = 0,   ///< инструкции кш
    Mil1553BcTypeOperations,         ///< операции кш
    Mil1553BcTypeData                ///< данные кш
};

/// \brief максимальный размер массива слов озу кш не данные
#define MIL1553_BC_MAX_SIZE_RAM_NONDATA     4096
/// \brief максимальный размер массива слов озу кш данные
#define MIL1553_BC_MAX_SIZE_RAM_DATA        (4096*2)

/// \brief ошибка записи в BC RAM - ошибка максимальной длины данных
#define MIL1553_BC_WRT_DATA_RAM_ERROR_MAX_LEN       -2
/// \brief ошибка записи в BC RAM - ошибка записи одного блока RAM
#define MIL1553_BC_WRT_DATA_RAM_ERROR_BAD_WRT       -3
/// \brief ошибка записи в BC RAM - ошибка чтения одного блока RAM
#define MIL1553_BC_WRT_DATA_RAM_ERROR_BAD_RD        -4
/// \brief ошибка записи в BC RAM - ошибка контроля записи одного блока RAM
#define MIL1553_BC_WRT_DATA_RAM_ERROR_BAD_CHECK     -5

/// \brief кол-ва слов в подадресе
#define MIL1553_RT_SUBADDRESS_COUNT_WORDS   32

///
/// \brief The TRtSubaddressData struct данные в подадресе
///
struct TRtSubaddressData {
    unsigned short mWords[MIL1553_RT_SUBADDRESS_COUNT_WORDS];   ///< 16-битные слова
    int mCount;                                                 ///< кол-во слов
};

/*-------------------------------------------------------------------------------------------------------*/

/// /// /// /// /// /// /// /// /// ///
/// \remark ОБЩИЕ СЕРВИСНЫЕ ФУНКЦИИ
/// /// /// /// /// /// /// /// /// ///

/// \brief невалидное значение, свидетельствующее об ошибке
#define LIB_BAD_VALUE       0xFFFFFFFF
/// \brief значение отсутсвия ошибки
#define LIB_GOOD            0

///
/// \brief макрос генериует имя регистра в соответствии с номером канала
/// \param numCh номер канала
/// \param RegName имя регистра без номера канала
///
/// \code
/// unsigned int numCh = mil1553ud_getNumberChannel(fd);
/// unsigned int regAddress = MIL_REG(numCh, CTRL_REG_PCI_CH);
///
#define MIL_REG(numCh, RegName) \
    ( (numCh == CH_NUM_1) ? RegName##1 : ( (numCh == CH_NUM_2) ? RegName##2 : ( (numCh == CH_NUM_3) ? RegName##3 : RegName##4) ) )


/// \remark РЕЖИМ КШ

/// \brief действие - останов программы
#define BC_PROGRAM_STOP     0x0u
/// \brief действие - старт программ
#define BC_PROGRAM_START    ~BC_PROGRAM_STOP

/// \remark РЕЖИМ КШ - РАБОТА С МИКРОПРОГРАММОЙ КШ - ИНСТРУКЦИИ, ОПЕРАЦИИ

#define XEQ         0x1
#define XQF         0x15
#define XQG         0x16
#define XFG         0x1E
#define JMP         0x2
#define CAL         0x3
#define RTN         0x4
#define IRQ         0x6
#define HLT         0x7
#define DLY         0x8
#define WFT         0x9
#define CFT         0xA
#define CMT         0xB
#define LTT         0xD
#define LTH         0x18
#define LFT         0xE
#define SFT         0xF
#define WMP         0x1B
#define WMI         0x1C
#define DSZ         0x1D
#define FLG         0xC

#define IDENT       0xA

#define ALWAYS      0xF

///
/// \brief формирование инструкции (32-разрядное слово)
/// \param instruction инструкция
/// \param condition условие
/// \param parameter параметр
///
/// \code
/// unsigned int dword = INSTRUCTION(XEQ, ALWAYS, 0);
///
inline uint32_t EXT_INSTRUCTION(uint32_t instruction, uint32_t condition, uint32_t parameter)
{
    uint32_t instruction_body = (( (instruction & 0x1F) << 26 ) | ( IDENT << 21 ) | ( (condition & 0x1F) << 16));
    uint32_t pbit = instruction_body;

    pbit ^= pbit >> 16;
    pbit ^= pbit >> 8;
    pbit ^= pbit >> 4;
    pbit ^= pbit >> 2;
    pbit ^= pbit >> 1;

    return ((~pbit) << 31) | instruction_body | (parameter & 0xFFFF);
}

#ifndef INSTRUCTION
#define INSTRUCTION(instruction, condition, parameter) EXT_INSTRUCTION(instruction, condition, parameter)
#endif

///
/// \brief нечётная часть операции (32-разрядное слово)
/// \param command команда КШ
/// \param timeout время до начала следующей операции
///
#define OPERATION_ODD_PART(command, timeout)    \
    (   \
    ( (command & 0xFFFF) << 16 ) | (timeout & 0xFFFF)  \
    )

///
/// \brief чётная часть операции (32-разрядное слово)
/// \param cw1 КС1
/// \param parameter параметр
///
#define OPERATION_EVEN_PART(cw1, parameter) \
    (   \
    ( (cw1 & 0xFFFF) << 16 ) | (parameter & 0xFFFF)  \
    )

///
/// \brief командное слово (ГОСТ 52070)
/// \param adrOu адрес ОУ
/// \param k разряд "приём/передача"
/// 0 - ОУ должен принимать СД
/// 1 - ОУ должен передавать СД
/// \param suba_mode подадрес или режим управления
/// \param cntDw_code число СД или код команды
///
#define COMMAND_WORD(adrOu, k, suba_mode, cntDw_code)  \
    (   \
    ( (adrOu & 0x1F) << 11 ) | ( (k & 0x1) << 10) | ( (suba_mode & 0x1F) << 5) | ( (cntDw_code & 0x1F) << 0)   \
    )

/// \remark РАСШИФРОВКА БЛОКОВ ДМА

/// \remark РАСШИФРОВКА БЛОКОВ ДМА - СЕРВИСНЫЕ КОНСТАНТЫ

/// \brief тип блока ДМА - КШ
#define MIL1553UD_BC_DMA_BLOCK      0
/// \brief тип блока ДМА - ОУ
#define MIL1553UD_TD_DMA_BLOCK      1

/// \brief значение FALSE (ложь)
#define MIL1553UD_FALSE             0
/// \brief значение TRUE (истина)
#define MIL1553UD_TRUE              ~(MIL1553UD_FALSE)

/// \brief размер двойного слова в байтах
#define DWORD_SIZE  4
/// \brief размер блока ДМА
#define MIL_DMA_BLOCK_SIZE 128

/// \remark РАСШИФРОВКА БЛОКОВ ДМА - СЕРВИСНЫЕ МАКРОСЫ

/// \brief получить указатель на блок ДМА
#define GET_PTR_DMA_BLOCK(data, numberBlock) \
    data + MIL_DMA_BLOCK_SIZE * numberBlock

/// \brief проверка на допустимость
#define ASSERT_DMA_CONTAINER(ptrContainerDma, numberBlock) \
    (ptrContainerDma == NULL) ? MIL1553UD_FALSE : \
        (numberBlock > ptrContainerDma->countBlocks) ? MIL1553UD_FALSE : MIL1553UD_TRUE

/// \brief Шина "А"
#define MIL1553_BUS_A       0x0
/// \brief Шина "Б"
#define MIL1553_BUS_B       ~MIL1553_BUS_A

#ifdef __cplusplus
}
#endif


#endif // MIL1553UD_COMMON_H
