#include "mil1553udext_eth_ioctl.h"
#include <stdarg.h>
#include <sys/types.h>
#include "mil1553udext_ioctl_wrappers.h"

#ifndef _WIN32
    #include <sys/ioctl.h>  /* ioctl */
    #include <sys/socket.h>
#endif

#ifndef MSG_NOSIGNAL
#define MSG_NOSIGNAL 0
#endif

#ifdef _WIN32
    #include <winsock2.h>
    #include <windows.h>
#else
    #include <sys/socket.h>
#endif

#include "mil1553udext_meu_proto.h"
#include "time.h"

#ifndef min
    #define min(X,Y) ((X) < (Y) ? (X) : (Y))
#endif

#ifndef max
    #define max(X,Y) ((X) > (Y) ? (X) : (Y))
#endif

/// \remark заголовки функций ioctl-команд

long eth_ioctl_simple(Mil1553ChannelFD cfd, const char *funcName, uint32_t meu_request, uint32_t meu_result);

long eth_ioctl_write_reg(Mil1553ChannelFD cfd, void *arg);
long eth_ioctl_write_reg_bit_mask(Mil1553ChannelFD cfd, void *arg);
long eth_ioctl_read_reg(Mil1553ChannelFD cfd, void *arg);

long eth_ioctl_switch(Mil1553ChannelFD cfd, void *arg, const char *funcName, int op_request, int op_answer);

long eth_ioctl_set_rt_tr_mode(Mil1553ChannelFD cfd, void *arg);
long eth_ioctl_set_rt_tr_buf_ready(Mil1553ChannelFD cfd, void *arg);
long eth_ioctl_rt_buf_man_en(Mil1553ChannelFD cfd, void *arg);

long eth_ioctl_version(Mil1553ChannelFD cfd, void *arg);
long eth_ioctl_version_driver(Mil1553ChannelFD cfd, void *arg);
long eth_ioctl_pci_location(Mil1553ChannelFD cfd, void *arg);
long eth_ioctl_eth_location(Mil1553ChannelFD cfd, void *arg);
long eth_ioctl_usb_location(Mil1553ChannelFD cfd, void *arg);

long eth_ioctl_get_nblock_raw_dma(Mil1553ChannelFD cfd, void *arg);
long eth_ioctl_read_blocks_raw_dma(unsigned int cmd, Mil1553ChannelFD cfd, void *arg);

long eth_ioctl_manage_interrupt(Mil1553ChannelFD cfd, void *arg);
long eth_ioctl_man_irq_subaddress_tr(Mil1553ChannelFD cfd, void *arg);
long eth_ioctl_man_irq_subaddress_rcv(Mil1553ChannelFD cfd, void *arg);

long eth_ioctl_set_address(Mil1553ChannelFD cfd, void *arg);
long eth_ioctl_set_timer(Mil1553ChannelFD cfd, void *arg, const char *funcName, int op_request, int op_answer);

long eth_ioctl_wr_block_buf_subadr(Mil1553ChannelFD cfd, void *arg);
long eth_ioctl_rd_block_buf_subadr(Mil1553ChannelFD cfd, void *arg);

long eth_ioctl_write_bc_ram(Mil1553ChannelFD cfd, void *arg);
long eth_ioctl_read_bc_ram(Mil1553ChannelFD cfd, void *arg);

long eth_ioctl_get_count_interrupt_blocks(Mil1553ChannelFD cfd, void *arg);
long eth_ioctl_read_interrupt_blocks(Mil1553ChannelFD cfd, void *arg);

long eth_ioctl_get_rcv_subadr_data(Mil1553ChannelFD cfd, void *arg);

long eth_ioctl_fw_begin(Mil1553ChannelFD cfd, void *arg);


long eth_ioctl_read_fw_status(Mil1553ChannelFD cfd, void *arg);
long eth_ioctl_fw_load(Mil1553ChannelFD cfd, void *arg);

long eth_ioctl_rd_settings(Mil1553ChannelFD cfd, void *arg);
long eth_ioctl_wr_settings(Mil1553ChannelFD cfd, void *arg);

typedef struct
{
    void *kernel_buffer;
    size_t kernel_buffer_size;

    void *user_buffer;
    size_t user_buffer_size;
} buffer_struct;

int chan_simple_send(Mil1553ChannelFD cfd, void *request, int op, void *answer, size_t answer_size);

int chan_send_with_userbuf(Mil1553ChannelFD cfd, buffer_struct request, int op, buffer_struct response);

///
/// \brief __log_func функция логирования в dmesg
/// \param lvl уровень
/// \param fmt строка
///
static void __log_func(const char* lvl, const char* fmt, ...)
{
    va_list args;
    va_start(args, fmt);
    printf("extmil1553 [%s]: ", lvl);
    vprintf(fmt, args);
    va_end(args);
}

#define copy_from_user 0; memcpy

#define copy_to_user 0; memcpy

#define mil_kzalloc(X, Y) malloc(X)

#define mil_kfree free


#ifdef DEBUG_CONFIGURATION

#ifndef LOG_LEVEL_INFO
/// \brief уровень "сообщение"
#define LOG_LEVEL_INFO "INFO"
#endif

#ifndef LOG_LEVEL_WARNING
/// \brief уровень "предупреждение"
#define LOG_LEVEL_WARNING "WARNING"
#endif

#ifndef LOG_INFO
/// \brief логирование сообщения
#define LOG_INFO(fmt, args...) \
    __log_func(LOG_LEVEL_INFO, fmt, ## args)
#endif

#ifndef LOG_WARNING
/// \brief логирование предупреждения
#define LOG_WARNING(fmt, args...) \
    __log_func(LOG_LEVEL_WARNING, fmt, ## args)
#endif

#else // DEBUG_CONFIGURATION



#ifndef LOG_INFO

#ifdef _WIN32

#define LOG_INFO(...)
#else

/// \brief логирование сообщения
#define LOG_INFO(fmt, args...)
#endif

#endif //LOG_INFO

#ifndef LOG_WARNING

#ifdef _WIN32

#define LOG_WARNING(...)
#else

/// \brief логирование предупреждения
#define LOG_WARNING(fmt, args...)
#endif

#endif //LOG_WARNING

#endif // DEBUG_CONFIGURATION

#ifndef LOG_LEVEL_ERROR
/// \brief уровень "ошибка"
#define LOG_LEVEL_ERROR "ERROR"
#endif

#ifndef LOG_ERROR
#ifdef _WIN32

/// \brief логирование ошибки
#define LOG_ERROR(...)
#else
/// \brief логирование ошибки
#define LOG_ERROR(fmt, args...) \
    __log_func(LOG_LEVEL_ERROR, fmt, ## args)
#endif

#endif //LOG_ERROR

#ifndef LOG_LEVEL_RELEASE_INFO
/// \brief уровень "инфо всегда"
#define LOG_LEVEL_RELEASE_INFO "RELEASE_INFO"
#endif

#ifndef LOG_RELEASE_INFO
#ifdef _WIN32

#define LOG_RELEASE_INFO(...)
#else

/// \brief логирование ошибки
#define LOG_RELEASE_INFO(fmt, args...) \
    __log_func(LOG_LEVEL_RELEASE_INFO, fmt, ## args)

#endif
#endif



long ext_ioctl_eth_handler(Mil1553ChannelFD cfd, unsigned int cmd, ...)
{
    void *arg = 0;
    long err = 0;

    va_list valist;

    LOG_INFO("fd=%d, chan=%d: cmd %d\n", cfd.socketFD, cfd.channel, cmd);

    if (_IOC_TYPE( cmd ) != IOC_MAGIC) {
        LOG_ERROR("ioctl_handler - bad IOC_MAGIC");
        return -ENOTTY;
    }

    if((cmd & (_IOC_DIRMASK << _IOC_DIRSHIFT)) != 0)
    {
        /* initialize valist for num number of arguments */
        va_start(valist, cmd);

        /* access all the arguments assigned to valist */
        // for (i = 0; i < num; i++)
        arg = va_arg(valist, void *);

        /* clean memory reserved for valist */
        va_end(valist);
    }

#ifdef DEBUG_CONFIGURATION
    char time_str[200];
    time_t t;
    struct tm *tmp;

    t = time(NULL);
    tmp = localtime(&t);
    if (tmp == NULL) {
        perror("localtime");
        exit(EXIT_FAILURE);
    }

    if (strftime(time_str, sizeof(time_str), "%c", tmp) == 0) {
        fprintf(stderr, "strftime returned 0");
        exit(EXIT_FAILURE);
    }

    printf("%s ext_ioctl_eth_handler : op %d\n", time_str, cmd & 0xFF);
#endif
    // ?? LOG_INFO("devIndex = %d, chNum = %d, drvIndex = %d", dev->index, dev->numCh, dev->drv->unit);

    switch (cmd) {
    // взаимодействие с регистрами
    case IOCTL_WRITE_REG:                       err = eth_ioctl_write_reg(cfd, arg);
        break;
    case IOCTL_WRITE_REG_BIT_MASK:              err = eth_ioctl_write_reg_bit_mask(cfd, arg);
        break;
    case IOCTL_READ_REG:                        err = eth_ioctl_read_reg(cfd, arg);
        break;

        // РЕЖИМЫ
    case IOCTL_SWITCH_MODE:
        // режим работы канала
        err =  eth_ioctl_switch(cfd, arg, "eth_ioctl_switch_mode", MEU_IOCTL_SWITCH_MODE_REQ, MEU_IOCTL_SWITCH_MODE_ANS);
        break;

    case IOCTL_SWITCH_BUS:
        // режим работы шины
        err = eth_ioctl_switch(cfd, arg, "eth_ioctl_switch_bus", MEU_IOCTL_SWITCH_BUS_REQ, MEU_IOCTL_SWITCH_BUS_ANS);
        break;

    case IOCTL_DEV_CHANNEL_WORK:
        // команда "разрешить работу канала"
        err = eth_ioctl_switch(cfd, arg, "ioctl_dev_channel_work", MEU_IOCTL_DEV_CHANNEL_WORK_REQ, MEU_IOCTL_DEV_CHANNEL_WORK_ANS);
        break;

        // term dev
    case IOCTL_SET_RT_TR_MODE:                  err = eth_ioctl_set_rt_tr_mode(cfd, arg);
        break;
    case IOCTL_SET_RT_TR_BUF_READY:             err = eth_ioctl_set_rt_tr_buf_ready(cfd, arg);
        break;
    case IOCTL_RT_BUF_MAN_EN:                   err = eth_ioctl_rt_buf_man_en(cfd, arg);
        break;
        // DMA

    case IOCTL_READ_BLOCKS_RAW_DMA:
    case IOCTL_READ_BLOCKS_RAW_DMA_FIX_COUNT:   err = eth_ioctl_read_blocks_raw_dma(cmd, cfd, arg);
        break;

    case IOCTL_GET_NBLOCK_RAW_DMA:              err = eth_ioctl_get_nblock_raw_dma(cfd, arg);
        break;
    case IOCTL_CLEAR_DMA:                       err = eth_ioctl_simple(cfd, "IOCTL_CLEAR_DMA", MEU_IOCTL_CLEAR_DMA_REQ, MEU_IOCTL_CLEAR_DMA_ANS);
        break;
    case IOCTL_ENABLE_DMA:                      err = eth_ioctl_simple(cfd, "IOCTL_ENABLE_DMA", MEU_IOCTL_ENABLE_DMA_REQ, MEU_IOCTL_ENABLE_DMA_ANS);
        break;
    case IOCTL_DISABLE_DMA:                     err = eth_ioctl_simple(cfd, "IOCTL_DISABLE_DMA", MEU_IOCTL_DISABLE_DMA_REQ, MEU_IOCTL_DISABLE_DMA_ANS);
        break;
        // IRQ
    case IOCTL_MANAGE_INTERRUPT:                err = eth_ioctl_manage_interrupt(cfd, arg);
        break;
    case IOCTL_MAN_IRQ_SUBADDRESS_TR:           err = eth_ioctl_man_irq_subaddress_tr(cfd, arg);
        break;
    case IOCTL_MAN_IRQ_SUBADDRESS_RCV:          err = eth_ioctl_man_irq_subaddress_rcv(cfd, arg);
        break;
        // КАНАЛЫ
    case IOCTL_SET_ADDRESS:                     err = eth_ioctl_set_address(cfd, arg);
        break;
    case IOCTL_SET_TIMER_RTBM_CONF_REG_PCI:
        err = eth_ioctl_set_timer(cfd, arg, "ioctl_set_timer_rtbm_conf_reg_pci", MEU_IOCTL_SET_TIMER_RTBM_CONF_REG_PCI_REQ, MEU_IOCTL_SET_TIMER_RTBM_CONF_REG_PCI_ANS);
        break;

    case IOCTL_SET_TIMER_BC_CONF_REG_PCI:
        err = eth_ioctl_set_timer(cfd, arg, "ioctl_set_timer_bc_conf_reg_pci", MEU_IOCTL_SET_TIMER_BC_CONF_REG_PCI_REQ, MEU_IOCTL_SET_TIMER_BC_CONF_REG_PCI_ANS);
        break;

        // ПОДАДРЕСА И БУФЕРЫ
    case IOCTL_WR_BLOCK_BUF_SUBADR:             err = eth_ioctl_wr_block_buf_subadr(cfd, arg);
        break;
    case IOCTL_RD_BLOCK_BUF_SUBADR:             err = eth_ioctl_rd_block_buf_subadr(cfd, arg);
        break;
        // bc_ram_reg
    case IOCTL_WRITE_BC_RAM:                    err = eth_ioctl_write_bc_ram(cfd, arg);
        break;
    case IOCTL_READ_BC_RAM:                     err = eth_ioctl_read_bc_ram(cfd, arg);
        break;
        // info command
    case IOCTL_VERSION:                         err = eth_ioctl_version(cfd, arg);
        break;
    case IOCTL_VERSION_DRIVER:                  err = eth_ioctl_version_driver(cfd, arg);
        break;

    case IOCTL_FW_STATUS:                       err = eth_ioctl_read_fw_status(cfd, arg);
        break;
    case IOCTL_FW_BEGIN:                        err = eth_ioctl_fw_begin(cfd, arg);
        break;
    case IOCTL_FW_LOAD:                         err = eth_ioctl_fw_load(cfd, arg);
        break;

    case IOCTL_FW_COUNT:                       err = eth_ioctl_simple(cfd, "IOCTL_FW_COUNT", MEU_IOCTL_FW_COUNT_REQ, MEU_IOCTL_FW_COUNT_ANS);
        break;
    case IOCTL_FW_UPDATE_APPLY:                err = eth_ioctl_simple(cfd, "IOCTL_FW_UPDATE_APPLY", MEU_IOCTL_FW_UPDATE_APPLY_REQ, MEU_IOCTL_FW_UPDATE_APPLY_ANS);
        break;
    case IOCTL_FW_UPDATE_CANCEL:               err = eth_ioctl_simple(cfd, "IOCTL_FW_UPDATE_CANCEL", MEU_IOCTL_FW_UPDATE_CANCEL_REQ, MEU_IOCTL_FW_UPDATE_CANCEL_ANS);
        break;

    case IOCTL_FW_RD_SETTINGS:                  err = eth_ioctl_rd_settings(cfd, arg);
        break;
    case IOCTL_FW_WR_SETTINGS:                  err = eth_ioctl_wr_settings(cfd, arg);
        break;


    case IOCTL_EXT_REBOOT:                       err = eth_ioctl_simple(cfd, "IOCTL_EXT_REBOOT", MEU_IOCTL_EXT_REBOOT_REQ, MEU_IOCTL_EXT_REBOOT_ANS);
        break;
    case IOCTL_EXT_GET_CURRENT:                  err = eth_ioctl_simple(cfd, "IOCTL_EXT_GET_CURRENT", MEU_IOCTL_EXT_GET_CURRENT_REQ, MEU_IOCTL_EXT_GET_CURRENT_ANS);
        break;

        // interrupt ring buffer
    case IOCTL_GET_COUNT_INTERRUPT_BLOCKS:      err = eth_ioctl_get_count_interrupt_blocks(cfd, arg);
        break;
    case IOCTL_READ_INTERRUPT_BLOCKS:           err = eth_ioctl_read_interrupt_blocks(cfd, arg);
        break;

    case IOCTL_GET_RCV_SUBADR_DATA:             err = eth_ioctl_get_rcv_subadr_data(cfd, arg);
        break;

    case IOCTL_EXT_GET_COUNT_CHANNELS:          return eth_ioctl_simple(cfd, "IOCTL_EXT_GET_COUNT_CHANNELS", MEU_OP_CH_COUNT_REQ, MEU_OP_CH_COUNT_ANS);
        break;
    default:
        LOG_ERROR("ioctl_handler - understand cmd - %d (%d)", cmd, cmd & 0xF);
        return -ENOTTY;
    }
    // обработка ошибок
    if (err)
    {
        #ifdef DEBUG_CONFIGURATION
            LOG_ERROR("mil1553cdev_ioctl ERROR: %ld [fd=%d, chan=%d] cmd %d (%d)\n", err, cfd.socketFD, cfd.channel, cmd, cmd & 0xFF);
            LOG_ERROR("ioctl_handler - error - %d", err);
        #endif
    }
    return err;
}


static void init_meu_header(meu_header *header, Mil1553ChannelFD cfd, int op, int size)
{
    header->magic = MEU_MAGIC;
    header->size  = size;
    header->chan  = cfd.channel;
    header->op    = op;
}


long eth_ioctl_simple(Mil1553ChannelFD cfd, const char *funcName, uint32_t meu_request, uint32_t meu_result)
{
    meu_header request;
    meu_simple_answer answer;
    long ret_code;

    LOG_INFO(funcName);

    init_meu_header(&request, cfd, meu_request, sizeof(request));

    ret_code = chan_simple_send(cfd, (void *) &request, meu_result, (void *) &answer, sizeof(answer));
    if(ret_code != 0)
        return ret_code;

    if(answer.ret != 0)
    {
        LOG_ERROR("answer.ret %ld\n", answer.ret);
        return answer.ret;
    }

    return ret_code;
}


/// \remark реализация ioctl-команд

long eth_ioctl_write_reg(Mil1553ChannelFD cfd, void *arg)
{
    meu_write_reg_request request;
    meu_simple_answer answer;
    SADDR_DATA saddr;
    int ret_code;

    LOG_INFO("ioctl_write_reg");
    ret_code = copy_from_user((void *) &saddr, (void *) arg, sizeof(SADDR_DATA));
    if(ret_code)
        return ret_code;

    init_meu_header(&request.header, cfd, MEU_IOCTL_WRITE_REG_REQ, sizeof(request));

    // Приходится перекладывать, непакованная структура
    request.daddr = saddr.daddr;
    request.data = saddr.data;

    ret_code = chan_simple_send(cfd, (void *) &request, MEU_IOCTL_WRITE_REG_ANS, (void *) &answer, sizeof(answer));
    if(ret_code == 0)
        return answer.ret;

    return ret_code;
}

long eth_ioctl_write_reg_bit_mask(Mil1553ChannelFD cfd, void *arg)
{
    int ret_code;
    meu_write_reg_bit_mask_request request;
    meu_simple_answer answer;
    SADDR_DATA_BIT_MASK data;
    LOG_INFO("ioctl_write_reg_bit_mask");

    ret_code = copy_from_user((void *) &data, (void *) arg, sizeof(SADDR_DATA_BIT_MASK));
    if(ret_code)
        return ret_code;

    init_meu_header(&request.header, cfd, MEU_IOCTL_WRITE_REG_BIT_MASK_REQ, sizeof(request));

    // Приходится перекладывать, непакованная структура
    request.bitmask.daddr = data.daddr;
    request.bitmask.data = data.data;
    request.bitmask.mask = data.mask;

    ret_code = chan_simple_send(cfd, (void *) &request, MEU_IOCTL_WRITE_REG_BIT_MASK_ANS, (void *) &answer, sizeof(answer));
    if(ret_code == 0)
        return answer.ret;

    return ret_code;
}

long eth_ioctl_read_reg(Mil1553ChannelFD cfd, void *arg)
{
    meu_read_reg_request request;
    meu_read_reg_answer answer;
    SADDR_DATA saddr;
    int ret_code;

    LOG_INFO("ioctl_write_reg");
    ret_code = copy_from_user((void *) &saddr, (void *) arg, sizeof(SADDR_DATA));
    if(ret_code)
        return ret_code;

    init_meu_header(&request.header, cfd, MEU_IOCTL_READ_REG_REQ, sizeof(request));

    // Приходится перекладывать, непакованная структура
    request.daddr = saddr.daddr;

    ret_code = chan_simple_send(cfd, (void *) &request, MEU_IOCTL_READ_REG_ANS, (void *) &answer, sizeof(answer));
    if(ret_code != 0)
        return ret_code;

    if(answer.ret != 0)
        return answer.ret;

    saddr.data = answer.data;
    ret_code = copy_to_user((void *) arg, (void *) &saddr, sizeof(SADDR_DATA));
    if(ret_code)
        return ret_code;

    return ret_code;
}

long eth_ioctl_switch(Mil1553ChannelFD cfd, void *arg, const char *funcName, int op_request, int op_result)
{
    unsigned int mode;          // режим работы
    int ret_code;
    meu_switch_request request;
    meu_simple_answer answer;

    LOG_INFO(funcName);
    ret_code = copy_from_user((void *) &mode, (void *) arg, sizeof(unsigned int));
    if(ret_code)
        return ret_code;

    init_meu_header(&request.header, cfd, op_request, sizeof(request));
    request.mode = mode;

    ret_code = chan_simple_send(cfd, (void *) &request, op_result, (void *) &answer, sizeof(answer));
    if(ret_code == 0)
        return answer.ret;

    return ret_code;
}


long eth_ioctl_set_rt_tr_mode(Mil1553ChannelFD cfd, void *arg)
{
    RT_TR_MODE data;            // режим работы ОУ

    int ret_code;
    meu_set_rttr_request request;
    meu_simple_answer answer;

    LOG_INFO("ioctl_set_rt_tr_mode");

    ret_code = copy_from_user((void *) &data, (void *) arg, sizeof(RT_TR_MODE));
    if(ret_code)
        return ret_code;

    init_meu_header(&request.header, cfd, MEU_IOCTL_SET_RT_TR_MODE_REQ, sizeof(request));
    request.mode = data.mode;
    request.subaddress = data.subaddress;

    ret_code = chan_simple_send(cfd, (void *) &request, MEU_IOCTL_SET_RT_TR_MODE_ANS, (void *) &answer, sizeof(answer));
    if(ret_code == 0)
        return answer.ret;

    return ret_code;
}

long eth_ioctl_set_rt_tr_buf_ready(Mil1553ChannelFD cfd, void *arg)
{
    RT_TR_MODE_BUF data;            // установить готовность буфера ОУ
    int ret_code;
    meu_set_rttr_request request;
    meu_simple_answer answer;

    LOG_INFO("ioctl_set_rt_tr_mode");

    ret_code = copy_from_user((void *) &data, (void *) arg, sizeof(RT_TR_MODE_BUF));
    if(ret_code)
        return ret_code;

    init_meu_header(&request.header, cfd, MEU_IOCTL_SET_RT_TR_BUF_READY_REQ, sizeof(request));
    request.mode = data.cmd_buf;
    request.subaddress = data.subaddress;

    ret_code = chan_simple_send(cfd, (void *) &request, MEU_IOCTL_SET_RT_TR_BUF_READY_ANS, (void *) &answer, sizeof(answer));
    if(ret_code == 0)
        return answer.ret;

    return ret_code;
}

long eth_ioctl_rt_buf_man_en(Mil1553ChannelFD cfd, void *arg)
{
    RT_BUF_MAN_EN data;            // разрешение буферов ОУ
    int ret_code;
    meu_set_rt_buf_man_request request;
    meu_simple_answer answer;

    LOG_INFO("ioctl_rt_buf_man_en");

    ret_code = copy_from_user((void *) &data, (void *) arg, sizeof(RT_BUF_MAN_EN));
    if(ret_code)
        return ret_code;

    init_meu_header(&request.header, cfd, MEU_IOCTL_RT_BUF_MAN_EN_REQ, sizeof(request));
    request.action_mask = data.action_mask;
    request.direction = data.direction;
    request.subaddress_mask = data.subaddress_mask;

    ret_code = chan_simple_send(cfd, (void *) &request, MEU_IOCTL_RT_BUF_MAN_EN_ANS, (void *) &answer, sizeof(answer));
    if(ret_code == 0)
        return answer.ret;

    return ret_code;
}

long eth_ioctl_version(Mil1553ChannelFD cfd, void *arg)
{
    int ret_code;
    VERSION version;    // подробная информация о драйвере и устройстве
    meu_header request;
    meu_version_answer answer;
    LOG_INFO("ioctl_version");

    init_meu_header(&request, cfd, MEU_IOCTL_VERSION_REQ, sizeof(request));

    ret_code = chan_simple_send(cfd, (void *) &request, MEU_IOCTL_VERSION_ANS, (void *) &answer, sizeof(answer));
    if(ret_code == 0)
    {
        if(answer.ret != 0)
            ret_code = answer.ret;
        else
        {
            // Приходится перекладывать, непакованная структура
            version.vendor_id = answer.version.vendor_id;
            version.device_id = answer.version.device_id;
            version.revision = answer.version.revision;
            version.type = answer.version.type;
            snprintf(version.dev_name, sizeof(version.dev_name), "%s", answer.version.dev_name);
            version.minor = answer.version.minor;
            version.irq = answer.version.irq;
            version.size_dma = answer.version.size_dma;
            version.addr_dma_virt = NULL; // dev->drv->dma_info.dma_vaddr;
            version.pci_bars = answer.version.pci_bars; // dev->drv->bar;
            version.addr_bar_virt = NULL; // dev->drv->pbar;

            ret_code = copy_to_user((void*)arg, (void*)&version, sizeof(VERSION));

            /*
             if (copy_to_user((void*)arg, (void*)&version, sizeof(VERSION)))
                return ERR_COPY_TO_USER;
            else
                return GOOD;
            */
        }
    }

    return ret_code;
}

long eth_ioctl_version_driver(Mil1553ChannelFD cfd, void *arg)
{
    int ret_code;
    unsigned int driverVersion;    // подробная информация о драйвере и устройстве
    meu_header request;
    meu_version_driver_answer answer;
    LOG_INFO("ioctl_version");

    init_meu_header(&request, cfd, MEU_IOCTL_VERSION_DRIVER_REQ, sizeof(request));

    ret_code = chan_simple_send(cfd, (void *) &request, MEU_IOCTL_VERSION_DRIVER_ANS, (void *) &answer, sizeof(answer));
    if(ret_code == 0)
    {
        if(answer.ret != 0)
            ret_code = answer.ret;
        else
        {
            driverVersion = answer.version;

            ret_code = copy_to_user((void*)arg, (void*)&driverVersion, sizeof(unsigned int));
            /*
            if (copy_to_user((void*)arg, (void*)&driverVersion, sizeof(unsigned int)))
                return ERR_COPY_TO_USER;
            else
                return GOOD;
*/
        }
    }

    return ret_code;
}

long eth_ioctl_get_nblock_raw_dma(Mil1553ChannelFD cfd, void *arg)
{
    meu_header request;
    meu_read_reg_answer answer;
    unsigned int nblock;
    int ret_code;

    LOG_INFO("ioctl_get_nblock_raw_dma");

    init_meu_header(&request, cfd, MEU_IOCTL_GET_NBLOCK_RAW_DMA_REQ, sizeof(request));

    ret_code = chan_simple_send(cfd, (void *) &request, MEU_IOCTL_GET_NBLOCK_RAW_DMA_ANS, (void *) &answer, sizeof(answer));
    if(ret_code != 0)
        return ret_code;

    if(answer.ret != 0)
    {
        LOG_ERROR("answer.ret %ld\n", answer.ret);
        return answer.ret;
    }

    nblock = answer.data;
    ret_code = copy_to_user((void *) arg, (void *) &nblock, sizeof(unsigned int));

    return ret_code;
}

long eth_ioctl_read_blocks_raw_dma(unsigned int cmd, Mil1553ChannelFD cfd, void *arg)
{
    meu_read_dma_request request;
    unsigned int nblock, checkNblock;
    int ret_code, data_size;

    DMA_READ_BLOCK block;       // данные из юзерспейс
    MIL1553_DMA_FIX_BLOCK fixBlock;

    meu_read_dma_answer answer;
    buffer_struct buf_request, buf_response;

    void *userBuffer;


    LOG_INFO("ioctl_read_blocks_raw_dma");

    if(cmd == IOCTL_READ_BLOCKS_RAW_DMA_FIX_COUNT)
    {
        ret_code = copy_from_user((void *) &fixBlock, (void *) arg, sizeof(fixBlock)); // получаем данные из юзерспейс
        request.n_blocks = fixBlock.mallocedBlocks;
        userBuffer = fixBlock.data;
    }
    else
    {
        ret_code = copy_from_user((void *) &block, (void *) arg, sizeof(block)); // получаем данные из юзерспейс
        request.n_blocks = block.countBlocks;
        userBuffer = block.data;
    }

    LOG_INFO("request.n_blocks = %d", request.n_blocks);

    if(ret_code)
        return ret_code;

    init_meu_header(&request.header, cfd, MEU_IOCTL_READ_BLOCKS_RAW_DMA_REQ, sizeof(request));
    data_size = request.n_blocks * MEU_SIZE_RAW_DMA_BLOCK;

    memset(&buf_request, 0, sizeof(buf_request));
    buf_request.kernel_buffer = (void *) &request;
    buf_request.kernel_buffer_size = sizeof(request);

    buf_response.kernel_buffer = (void *) &answer;
    buf_response.kernel_buffer_size = sizeof(answer);
    buf_response.user_buffer = userBuffer;
    buf_response.user_buffer_size = data_size;

    LOG_INFO("request.n_blocks = %d, buf_response.user_buffer_size = %d\n", request.n_blocks, data_size);

    // внутри идёт приём данных в юзерспейс через промежуточный буфер
    ret_code = chan_send_with_userbuf(cfd, buf_request, MEU_IOCTL_READ_BLOCKS_RAW_DMA_ANS, buf_response);
    if(ret_code != 0)
        goto cleanup_and_return;

    nblock = 0;
    if(answer.ret < 0)
    {
        ret_code = answer.ret;
        goto cleanup_and_return;
    }

    nblock = answer.n_currentBlocks;
    checkNblock = (answer.header.size - sizeof(meu_read_dma_answer)) / MEU_SIZE_RAW_DMA_BLOCK;

    if(checkNblock != answer.n_currentBlocks)
    {
        LOG_ERROR("nblock mismatch: nblock %d, received %d blocks", nblock, checkNblock);
    }

    // Фактическое количество полученных блоков
    if(cmd == IOCTL_READ_BLOCKS_RAW_DMA_FIX_COUNT)
    {
        fixBlock.currentBlocks = answer.n_currentBlocks;
        fixBlock.nextBlocks = answer.n_nextBlocks;

        ret_code = copy_to_user((void*)arg, (void*)&fixBlock, sizeof(MIL1553_DMA_FIX_BLOCK));
    }
    else
    {
        ret_code = copy_to_user((void*)arg, (void*)&nblock, sizeof(unsigned int));
    }

cleanup_and_return:
    return ret_code;
}

/// \brief управление битами прерывания
#define INTERRUPT_MANAGE(intState, shift, regVal)   \
    switch (intState) { \
    case INTERRUPT_MAN_ON:      regVal |= 1<<shift;    break; \
    case INTERRUPT_MAN_OFF:     regVal &= ~(1<<shift); break; \
    }

long eth_ioctl_manage_interrupt(Mil1553ChannelFD cfd, void *arg)
{
    return -EINVAL; // eth_ioctl_manage_interrupt
    if(0)
    {
#if 0
        INTERRUPT_MAN intMan;       // входные данные по управлению прерыванием
        unsigned int intRegMask;    // регистр INTERRUPT_MASK
        LOG_INFO("ioctl_manage_interrupt");
        copy_from_user((void *) &intMan, (void *) arg, sizeof(INTERRUPT_MAN));
        mutex_lock(&dev->drv->mtx);
        intRegMask = ioread32(dev->drv->pbar + INTERRUPT_MASK);
        // маскируем соответствующие прерывания
        INTERRUPT_MANAGE(intMan.int_flash, 24, intRegMask); // INT_FLASH
        switch (dev->numCh) {
        case CH_NUM_1: {
            INTERRUPT_MANAGE(intMan.int_bc,            10, intRegMask); // INT_BC
            INTERRUPT_MANAGE(intMan.int_rt_ren,         9, intRegMask); // INT_RT_REN
            INTERRUPT_MANAGE(intMan.int_rt_ten,         8, intRegMask); // INT_RT_TEN
            INTERRUPT_MANAGE(intMan.int_qdat,           1, intRegMask); // INT_QDAT
            INTERRUPT_MANAGE(intMan.int_hdat,           0, intRegMask); // INT_HDAT
            INTERRUPT_MANAGE(intMan.int_data_cnt_en,   11, intRegMask); // INT_DATA_CNT_EN
            INTERRUPT_MANAGE(intMan.int_timeout_itven, 24, intRegMask); // INT_TIMEOUT_ITVEN
            INTERRUPT_MANAGE(intMan.int_timeout_absen, 28, intRegMask); // INT_TIMEOUT_ABSEN
        } break;
        case CH_NUM_2: {
            INTERRUPT_MANAGE(intMan.int_bc,            14, intRegMask); // INT_BC
            INTERRUPT_MANAGE(intMan.int_rt_ren,        13, intRegMask); // INT_RT_REN
            INTERRUPT_MANAGE(intMan.int_rt_ten,        12, intRegMask); // INT_RT_TEN
            INTERRUPT_MANAGE(intMan.int_qdat,           3, intRegMask); // INT_QDAT
            INTERRUPT_MANAGE(intMan.int_hdat,           2, intRegMask); // INT_HDAT
            INTERRUPT_MANAGE(intMan.int_data_cnt_en,   15, intRegMask); // INT_DATA_CNT_EN
            INTERRUPT_MANAGE(intMan.int_timeout_itven, 25, intRegMask); // INT_TIMEOUT_ITVEN
            INTERRUPT_MANAGE(intMan.int_timeout_absen, 29, intRegMask); // INT_TIMEOUT_ABSEN
        } break;
        case CH_NUM_3: {
            INTERRUPT_MANAGE(intMan.int_bc,            18, intRegMask); // INT_BC
            INTERRUPT_MANAGE(intMan.int_rt_ren,        17, intRegMask); // INT_RT_REN
            INTERRUPT_MANAGE(intMan.int_rt_ten,        16, intRegMask); // INT_RT_TEN
            INTERRUPT_MANAGE(intMan.int_qdat,           5, intRegMask); // INT_QDAT
            INTERRUPT_MANAGE(intMan.int_hdat,           4, intRegMask); // INT_HDAT
            INTERRUPT_MANAGE(intMan.int_data_cnt_en,   19, intRegMask); // INT_DATA_CNT_EN
            INTERRUPT_MANAGE(intMan.int_timeout_itven, 26, intRegMask); // INT_TIMEOUT_ITVEN
            INTERRUPT_MANAGE(intMan.int_timeout_absen, 30, intRegMask); // INT_TIMEOUT_ABSEN
        } break;
        case CH_NUM_4: {
            INTERRUPT_MANAGE(intMan.int_bc,            22, intRegMask); // INT_BC
            INTERRUPT_MANAGE(intMan.int_rt_ren,        21, intRegMask); // INT_RT_REN
            INTERRUPT_MANAGE(intMan.int_rt_ten,        20, intRegMask); // INT_RT_TEN
            INTERRUPT_MANAGE(intMan.int_qdat,           7, intRegMask); // INT_QDAT
            INTERRUPT_MANAGE(intMan.int_hdat,           6, intRegMask); // INT_HDAT
            INTERRUPT_MANAGE(intMan.int_data_cnt_en,   23, intRegMask); // INT_DATA_CNT_EN
            INTERRUPT_MANAGE(intMan.int_timeout_itven, 27, intRegMask); // INT_TIMEOUT_ITVEN
            INTERRUPT_MANAGE(intMan.int_timeout_absen, 31, intRegMask); // INT_TIMEOUT_ABSEN
        } break;
        default:
            LOG_ERROR("bad channel");
            mutex_unlock(&dev->drv->mtx);
            return ERR_BAD_CHANNEL;
        }
        iowrite32(intRegMask, dev->drv->pbar + INTERRUPT_MASK);
        mutex_unlock(&dev->drv->mtx);
        return GOOD;
#endif
    }
}

/// \brief управление битами прерывания по подадресам поканально
#define SUBADDRESS_INTERRUPT_MANAGE(irq_sa, rtIntReg, rtIntRegAdr, nBitInt, rtReg, rtRegAdr, nBitRt, sa)   \
    rtIntReg = ioread32(dev->drv->pbar + rtIntRegAdr); \
    if (irq_sa.rt_int == INTERRUPT_MAN_ON) { rtIntReg |= 1 << nBitInt; } \
    else  if (irq_sa.rt_int == INTERRUPT_MAN_OFF) { rtIntReg &= ~(1 << nBitInt); } \
    iowrite32(rtIntReg, dev->drv->pbar + rtIntRegAdr); \
    for ( sa = 1; sa <= 30; sa++ ) { \
    rtReg = ioread32(dev->drv->pbar + rtRegAdr + sa * 0x4); \
    if (irq_sa.subaddress_mask & (1<<sa)) \
    rtReg |= 1 << nBitRt; \
    else \
    rtReg &= ~(1 << nBitRt); \
    iowrite32(rtReg, dev->drv->pbar + rtRegAdr + sa * 0x4); \
    }

long eth_ioctl_man_irq_subaddress_tr(Mil1553ChannelFD cfd, void *arg)
{
    return -EINVAL; // eth_ioctl_man_irq_subaddress_tr
    if(0)
    {
#if 0
        INTERRUPT_SUBADDRESS irq_sa;    // входные данные по управлению прерыванием
        unsigned int rtIntReg;          // регистр RT_INT_REG
        unsigned int rtReg;             // регистр RT_TR_REG
        unsigned int sa;                // текущий подадрес
        LOG_INFO("ioctl_man_irq_subaddress_tr");
        copy_from_user((void *) &irq_sa, (void *) arg, sizeof(INTERRUPT_SUBADDRESS)); // получаем данные из юзерспейс
        mutex_lock(&dev->mtxCh);
        switch (dev->numCh) {  // управляем прерыванием на передачу
        case CH_NUM_1:
            SUBADDRESS_INTERRUPT_MANAGE(irq_sa, rtIntReg, RT_INT_REG_CH1, 0, rtReg, RT_TR_REG_CH1, 24, sa);
            break;
        case CH_NUM_2:
            SUBADDRESS_INTERRUPT_MANAGE(irq_sa, rtIntReg, RT_INT_REG_CH2, 0, rtReg, RT_TR_REG_CH2, 24, sa);
            break;
        case CH_NUM_3:
            SUBADDRESS_INTERRUPT_MANAGE(irq_sa, rtIntReg, RT_INT_REG_CH3, 0, rtReg, RT_TR_REG_CH3, 24, sa);
            break;
        case CH_NUM_4:
            SUBADDRESS_INTERRUPT_MANAGE(irq_sa, rtIntReg, RT_INT_REG_CH4, 0, rtReg, RT_TR_REG_CH4, 24, sa);
            break;
        default:
            LOG_ERROR("bad channel");
            mutex_unlock(&dev->mtxCh);
            return ERR_BAD_CHANNEL;
        }
        mutex_unlock(&dev->mtxCh);
        return GOOD;
#endif
    }
}

long eth_ioctl_man_irq_subaddress_rcv(Mil1553ChannelFD cfd, void *arg)
{
    return -EINVAL; // eth_ioctl_man_irq_subaddress_rcv
    if(0)
    {
#if 0
        INTERRUPT_SUBADDRESS irq_sa;    // входные данные по управлению прерыванием
        unsigned int rtIntReg;          // регистр RT_INT_REG
        unsigned int rtReg;             // регистр RT_TR_REG
        unsigned int sa;                // текущий подадрес
        LOG_INFO("ioctl_man_irq_subaddress_rcv");
        copy_from_user((void *) &irq_sa, (void *) arg, sizeof(INTERRUPT_SUBADDRESS)); // получаем данные из юзерспейс
        mutex_lock(&dev->mtxCh);
        switch (dev->numCh) { // управляем прерыванием на приём
        case CH_NUM_1:
            SUBADDRESS_INTERRUPT_MANAGE(irq_sa, rtIntReg, RT_INT_REG_CH1, 1, rtReg, RT_RCV_REG_CH1, 29, sa);
            break;
        case CH_NUM_2:
            SUBADDRESS_INTERRUPT_MANAGE(irq_sa, rtIntReg, RT_INT_REG_CH2, 1, rtReg, RT_RCV_REG_CH2, 29, sa);
            break;
        case CH_NUM_3:
            SUBADDRESS_INTERRUPT_MANAGE(irq_sa, rtIntReg, RT_INT_REG_CH3, 1, rtReg, RT_RCV_REG_CH3, 29, sa);
            break;
        case CH_NUM_4:
            SUBADDRESS_INTERRUPT_MANAGE(irq_sa, rtIntReg, RT_INT_REG_CH4, 1, rtReg, RT_RCV_REG_CH4, 29, sa);
            break;
        default:
            LOG_ERROR("bad channel");
            mutex_unlock(&dev->mtxCh);
            return ERR_BAD_CHANNEL;
        }
        mutex_unlock(&dev->mtxCh);
        return GOOD;
#endif
    }
}

long eth_ioctl_set_address(Mil1553ChannelFD cfd, void *arg)
{
    unsigned char address;      // адрес ОУ (1-30)
    meu_set_addr_request request;
    meu_simple_answer answer;
    int ret_code;

    LOG_INFO("ioctl_set_address");
    ret_code = copy_from_user((void *) &address, (void *) arg, sizeof(unsigned char)); // получаем данные из юзерспейс
    if(ret_code)
        return ret_code;

    init_meu_header(&request.header, cfd, MEU_IOCTL_SET_ADDRESS_REQ, sizeof(request));
    request.daddr = address;

    ret_code = chan_simple_send(cfd, (void *) &request, MEU_IOCTL_SET_ADDRESS_ANS, (void *) &answer, sizeof(answer));
    if(ret_code != 0)
        return ret_code;

    if(answer.ret != 0)
        return answer.ret;

    return ret_code;
}

long eth_ioctl_set_timer(Mil1553ChannelFD cfd, void *arg, const char *funcName, int op_request, int op_answer)
{
    TIMER_TIMEOUT timeout;
    meu_set_timer_request request;
    meu_simple_answer answer;
    int ret_code;

    LOG_INFO("ioctl_set_timer_rtbm_conf_reg_pci");
    ret_code = copy_from_user((void *) &timeout, (void *) arg, sizeof(timeout)); // получаем данные из юзерспейс
    if(ret_code)
        return ret_code;

    init_meu_header(&request.header, cfd, op_request, sizeof(request));
    request.rcvck = timeout.rcvck;
    request.tck = timeout.tck;
    request.trck = timeout.trck;

    ret_code = chan_simple_send(cfd, (void *) &request, op_answer, (void *) &answer, sizeof(answer));
    if(ret_code != 0)
        return ret_code;

    if(answer.ret != 0)
        return answer.ret;

    return ret_code;
}

long eth_ioctl_wr_block_buf_subadr(Mil1553ChannelFD cfd, void *arg)
{
    RT_TR_BUF_SUBADDR buf_sa;
    meu_write_rttr_subaddr_request request;
    meu_simple_answer answer;
    int ret_code;

    LOG_INFO("ioctl_wr_block_buf_subadr");
    ret_code = copy_from_user((void *) &buf_sa, (void *) arg, sizeof(buf_sa)); // получаем данные из юзерспейс
    if(ret_code)
        return ret_code;

    init_meu_header(&request.header, cfd, MEU_IOCTL_WR_BLOCK_BUF_SUBADR_REQ, sizeof(request));
    memcpy(request.data_words, buf_sa.data_words, MEU_RTTR_SUBADDR_DATA_WORDS_COUNT * sizeof(uint16_t));
    request.num_buf = buf_sa.num_buf;
    request.subaddress = buf_sa.subaddress;

    ret_code = chan_simple_send(cfd, (void *) &request, MEU_IOCTL_WR_BLOCK_BUF_SUBADR_ANS, (void *) &answer, sizeof(answer));
    if(ret_code != 0)
        return ret_code;

    if(answer.ret != 0)
        return answer.ret;

    return ret_code;
}

long eth_ioctl_rd_block_buf_subadr(Mil1553ChannelFD cfd, void *arg)
{
    RT_TR_BUF_SUBADDR bufSa;        // подадрес
    meu_read_rttr_subaddr_request request;
    meu_read_rttr_subaddr_answer answer;
    int ret_code;

    LOG_INFO("eth_ioctl_rd_block_buf_subadr");
    ret_code = copy_from_user((void *) &bufSa, (void *) arg, sizeof(bufSa));
    if(ret_code)
        return ret_code;

    init_meu_header(&request.header, cfd, MEU_IOCTL_RD_BLOCK_BUF_SUBADR_REQ, sizeof(request));

    // Приходится перекладывать, непакованная структура
    request.num_buf = bufSa.num_buf;
    request.subaddress = bufSa.subaddress;

    ret_code = chan_simple_send(cfd, (void *) &request, MEU_IOCTL_RD_BLOCK_BUF_SUBADR_ANS, (void *) &answer, sizeof(answer));
    if(ret_code != 0)
        return ret_code;

    if(answer.ret != 0)
        return answer.ret;

    memcpy(bufSa.data_words, answer.data_words, MEU_RTTR_SUBADDR_DATA_WORDS_COUNT * sizeof(uint16_t));

    return copy_to_user((void *) arg, (void *) &bufSa, sizeof(bufSa));
}

#define MEU_CONVERT_BLOCK \
    dest->is_protected=src->is_protected; /* [out] Разрешено ли обновление в этот раздел */  \
    dest->is_static_ip_enabled=src->is_static_ip_enabled; /* если false, адрес будет взят по dhcp. Если true - из поля ниже */ \
    dest->is_netmask_set=src->is_netmask_set; /* если false, шлюз не учитывается */ \
    \
    for(dest->block_id = 0; dest->block_id < 6; dest->block_id++) \
    dest->mac_addr[ dest->block_id ]=src->mac_addr[ dest->block_id ]; /* мак-адрес */ \
    \
    for(dest->block_id = 0; dest->block_id < 4; dest->block_id++) \
{ \
    dest->ipv4_addr[ dest->block_id ]=src->ipv4_addr[ dest->block_id ];  /* адрес интерфейса */ \
    dest->ipv4_netmask[ dest->block_id ]=src->ipv4_netmask[ dest->block_id ];  /* сетевая маска */ \
    dest->ipv4_gateway[ dest->block_id ]=src->ipv4_gateway[ dest->block_id ];  /* шлюз */ \
    } \
    \
    for(dest->block_id = 0; dest->block_id < MEU_CONF_BLOCK_UPDATE_DESC_SIZE; dest->block_id++) \
    dest->update_desc[ dest->block_id ]=src->update_desc[ dest->block_id ];  /* строка с описанием обновления (ascii латиница) */ \
    \
    dest->is_gateway_set=src->is_gateway_set;  /* если false, шлюз не учитывается */ \
    dest->mil_serial_num=src->mil_serial_num;  /* Уникальный идентификатор устройства в серии (для различения нескольких usb) */ \
    dest->server_port=src->server_port;  /* порт для внутреннего ПО mil-eth-server */ \
    dest->active_block_id=src->active_block_id;  /* Индекс раздела обновления */ \
    dest->block_id=src->block_id  /* Индекс раздела обновления */ \

void fromUserConfToMeu(CONF_BLOCK *src, meu_conf_block *dest)
{
    MEU_CONVERT_BLOCK;
}

void fromMeuToUserConf(meu_conf_block *src, CONF_BLOCK *dest)
{
    MEU_CONVERT_BLOCK;
}

/*
 @brief Запрос: начать процесс обновления
 Идёт с помощью meu_io_conf_wr_request

 @brief Ответ: начать процесс обновления
 Идёт с помощью meu_simple_answer
 */
long eth_ioctl_fw_begin(Mil1553ChannelFD cfd, void *arg)
{
    meu_fw_begin_request request;
    meu_simple_answer answer;
    long ret_code;
    CONF_BLOCK conf;

    LOG_INFO("eth_ioctl_begin_fw");
    ret_code = copy_from_user((void *) &conf, (void *) arg, sizeof(conf)); // получаем данные из юзерспейс
    if(ret_code)
        return ret_code;

    fromUserConfToMeu(&conf, &request.conf);
    init_meu_header(&request.header, cfd, MEU_IOCTL_FW_BEGIN_REQ, sizeof(request));

    LOG_INFO("eth_ioctl_begin_fw: op %d, sz %d, block %d\n", request.header.op, request.header.size, request.conf.block_id);

    ret_code = chan_simple_send(cfd, (void *) &request, MEU_IOCTL_FW_BEGIN_ANS, (void *) &answer, sizeof(answer));
    if(ret_code != 0)
        return ret_code;

    if(answer.ret != 0)
    {
        LOG_ERROR("answer.ret %ld\n", answer.ret);
        return answer.ret;
    }

    return ret_code;
}

long eth_ioctl_read_fw_status(Mil1553ChannelFD cfd, void *arg)
{
    FW_STATUS status;        // подадрес
    meu_header request;
    meu_fw_status_answer answer;
    int ret_code;

    LOG_INFO("eth_ioctl_read_fw_status");

    init_meu_header(&request, cfd, MEU_IOCTL_FW_STATUS_REQ, sizeof(request));

    ret_code = chan_simple_send(cfd, (void *) &request, MEU_IOCTL_FW_STATUS_ANS, (void *) &answer, sizeof(answer));
    if(ret_code != 0)
        return ret_code;

    if(answer.ret != 0)
        return answer.ret;

    // Приходится перекладывать, непакованная структура
    status.stage = answer.stage;
    status.percentComplete = answer.percentComplete;

    return copy_to_user((void *) arg, (void *) &status, sizeof(status));
}

long eth_ioctl_fw_load(Mil1553ChannelFD cfd, void *arg)
{
    FW_BLOCK block; // данные из юзерспейс
    meu_simple_answer answer;
    meu_io_fw_request request;
    buffer_struct buf_request, buf_response;
    int ret_code;

    LOG_INFO("eth_ioctl_write_fw_block - begin\n");
    LOG_INFO("eth_ioctl_write_fw_block");
    ret_code = copy_from_user((void *) &block, (void *) arg, sizeof(block)); // получаем данные из юзерспейс
    if(ret_code)
        return ret_code;

    if(block.blockSize == 0)
        return 0;

    if(block.fileSize == 0)
        return -EINVAL;

    memset(&buf_request, 0, sizeof(buf_request));
    buf_request.kernel_buffer = &request;
    buf_request.kernel_buffer_size = sizeof(request);
    buf_request.user_buffer = block.binaryBlock;
    buf_request.user_buffer_size = block.blockSize;

    memset(&buf_response, 0, sizeof(buf_response));
    buf_response.kernel_buffer = &answer;
    buf_response.kernel_buffer_size = sizeof(answer);

    init_meu_header(&request.header, cfd, MEU_IOCTL_FW_LOAD_REQ, buf_request.kernel_buffer_size + buf_request.user_buffer_size);
    request.blockOffset = block.blockOffset;
    request.blockSize = block.blockSize;
    request.fileSize = block.fileSize;

    ret_code = chan_send_with_userbuf(cfd, buf_request, MEU_IOCTL_FW_LOAD_ANS, buf_response);

    if(ret_code == 0)
        ret_code = answer.ret;

    LOG_INFO("eth_ioctl_write_fw_block - end\n");
    return ret_code;
}


/**
 Запрос: скачать настройки ПО lan1553ud
 Идёт с помощью meu_io_conf_rd_request

 Ответ: скачать настройки ПО lan1553ud
 Идёт с помощью meu_io_conf_result
 */
long eth_ioctl_rd_settings(Mil1553ChannelFD cfd, void *arg)
{
    CONF_BLOCK block;             // данные из юзерспейс
    meu_io_conf_rd_request request;
    meu_io_conf_result answer;
    int ret_code;


    LOG_INFO("eth_ioctl_read_settings - begin\n");
    LOG_INFO("eth_ioctl_read_settings");
    ret_code = copy_from_user((void *) &block, (void *) arg, sizeof(block)); // получаем данные из юзерспейс
    if(ret_code)
        return ret_code;

    init_meu_header(&request.header, cfd, MEU_IOCTL_FW_RD_SETTINGS_REQ, sizeof(request));

    request.block_id = block.block_id;
    LOG_INFO("eth_ioctl_read_settings : block_id %d\n", request.block_id);

    ret_code = chan_simple_send(cfd, (void *) &request, MEU_IOCTL_FW_RD_SETTINGS_ANS, &answer, sizeof(answer));

    if(ret_code == 0)
    {
        LOG_INFO("eth_ioctl_read_settings - ret 0\n");
        ret_code = answer.ret;
        if(ret_code == 0)
        {
            fromMeuToUserConf(&answer.conf, &block);
            ret_code = copy_to_user((void *) arg, (void *) &block, sizeof(block));
        }
    }

    LOG_INFO("eth_ioctl_read_settings - end ret: %d\n", ret_code);
    return ret_code;
}


/*
 Запрос: загрузить настройки ПО lan1553ud
 Идёт с помощью meu_io_conf_wr_request

 Ответ: загрузить настройки ПО lan1553ud
 Идёт с помощью meu_simple_answer
 */
long eth_ioctl_wr_settings(Mil1553ChannelFD cfd, void *arg)
{
    CONF_BLOCK conf;             // данные из юзерспейс
    meu_simple_answer answer;
    meu_io_conf_wr_request request;
    int ret_code;

    LOG_INFO("eth_ioctl_write_settings - begin\n");
    LOG_INFO("eth_ioctl_write_settings");
    ret_code = copy_from_user((void *) &conf, (void *) arg, sizeof(conf)); // получаем данные из юзерспейс
    if(ret_code)
        return ret_code;

    init_meu_header(&request.header, cfd, MEU_IOCTL_FW_WR_SETTINGS_REQ, sizeof(request));
    fromUserConfToMeu(&conf, &request.conf);

    ret_code = chan_simple_send(cfd, &request, MEU_IOCTL_FW_WR_SETTINGS_ANS, (void *) &answer, sizeof(answer));

    if(ret_code == 0)
        ret_code = answer.ret;

    LOG_INFO("eth_ioctl_write_settings - end ret: %d\n", ret_code);
    return ret_code;
}


long eth_ioctl_write_bc_ram(Mil1553ChannelFD cfd, void *arg)
{
    BC_RAM_BLOCK block;             // данные из юзерспейс
    meu_simple_answer answer;
    void *raw_request;
    meu_io_bc_ram_request *request;
    int ret_code, raw_request_size, data_byte_count;

    LOG_INFO("eth_ioctl_write_bc_ram - begin\n");
    LOG_INFO("eth_ioctl_write_bc_ram");
    ret_code = copy_from_user((void *) &block, (void *) arg, sizeof(block)); // получаем данные из юзерспейс
    if(ret_code)
        return ret_code;

    if(block.length == 0)
        return 0;

    data_byte_count = block.length * sizeof(uint32_t);
    raw_request_size = sizeof(meu_io_bc_ram_request) + data_byte_count;
    raw_request = mil_kzalloc(raw_request_size, GFP_KERNEL);
    if(raw_request == NULL)
        return -ENOMEM;
    request = (meu_io_bc_ram_request *) raw_request;

    init_meu_header(&request->header, cfd, MEU_IOCTL_WRITE_BC_RAM_REQ, raw_request_size);
    request->length = block.length;
    request->shift = block.shift;
    request->type = block.type;

    ret_code = copy_from_user((void *)(sizeof(meu_io_bc_ram_request) + (char *) raw_request),
                              (void *) block.dwords, data_byte_count); // получаем данные из юзерспейс

    if(ret_code == 0)
    {
        ret_code = chan_simple_send(cfd, raw_request, MEU_IOCTL_WRITE_BC_RAM_ANS, (void *) &answer, sizeof(answer));

        if(ret_code == 0)
            ret_code = answer.ret;
    }
    else
    {
        LOG_INFO("eth_ioctl_write_bc_ram - copy_from_user ret: %d\n", ret_code);
    }

    mil_kfree(raw_request);
    LOG_INFO("eth_ioctl_write_bc_ram - end ret: %d\n", ret_code);
    return ret_code;
}

long eth_ioctl_read_bc_ram(Mil1553ChannelFD cfd, void *arg)
{
    BC_RAM_BLOCK block;             // данные из юзерспейс
    meu_io_bc_ram_request request;
    void *raw_answer;
    meu_simple_answer *answer;
    int ret_code, raw_answer_size, data_byte_count;


    LOG_INFO("eth_ioctl_read_bc_ram - begin\n");
    LOG_INFO("eth_ioctl_read_bc_ram");
    ret_code = copy_from_user((void *) &block, (void *) arg, sizeof(block)); // получаем данные из юзерспейс
    if(ret_code)
        return ret_code;

    if(block.length == 0)
        return 0;

    data_byte_count = block.length * sizeof(uint32_t);
    raw_answer_size = sizeof(meu_simple_answer) + data_byte_count;
    raw_answer = mil_kzalloc(raw_answer_size, GFP_KERNEL);
    if(raw_answer == NULL)
        return -ENOMEM;
    answer = (meu_simple_answer *) raw_answer;

    init_meu_header(&request.header, cfd, MEU_IOCTL_READ_BC_RAM_REQ, sizeof(request));
    request.length = block.length;
    request.shift = block.shift;
    request.type = block.type;

    LOG_INFO("length: %d\n", block.length);
    LOG_INFO("shift: %d\n", block.shift);
    LOG_INFO("type: %d\n", block.type);

    ret_code = chan_simple_send(cfd, (void *) &request, MEU_IOCTL_READ_BC_RAM_ANS, raw_answer, raw_answer_size);

    if(ret_code == 0)
    {
        LOG_INFO("eth_ioctl_read_bc_ram - ret 0\n");
        ret_code = answer->ret;
        if(ret_code == 0)
        {
            LOG_INFO("eth_ioctl_read_bc_ram - recv ret 0\n");
            ret_code = copy_to_user((void *) block.dwords, //
                                    (void *)(sizeof(meu_simple_answer) + (char *) raw_answer), //
                                    data_byte_count); // кладём данные в юзерспейс

            LOG_INFO("eth_ioctl_read_bc_ram - copy to user ret: %d\n", ret_code);
        }
    }

    mil_kfree(raw_answer);


    LOG_INFO("eth_ioctl_read_bc_ram - end ret: %d\n", ret_code);
    return ret_code;
}

long eth_ioctl_get_count_interrupt_blocks(Mil1553ChannelFD cfd, void *arg)
{
    meu_header request;
    meu_read_reg_answer answer;
    int ret_code;
    unsigned int count_blocks;

    LOG_INFO("ioctl_get_count_interrupt_blocks");

    init_meu_header(&request, cfd, MEU_IOCTL_GET_COUNT_INTERRUPT_BLOCKS_REQ, sizeof(request));

    ret_code = chan_simple_send(cfd, (void *) &request, MEU_IOCTL_GET_COUNT_INTERRUPT_BLOCKS_ANS, (void *) &answer, sizeof(answer));
    if(ret_code == 0)
    {
        ret_code = answer.ret;
        if(ret_code == 0)
        {
            count_blocks = answer.data;
            ret_code = copy_to_user((void*)arg, (void*)&count_blocks, sizeof(unsigned int));
        }
    }

    return ret_code;
}

long eth_ioctl_read_interrupt_blocks(Mil1553ChannelFD cfd, void *arg)
{
    meu_read_intblocks_request request;
    meu_read_reg_answer *answer;
    meu_intblock_info *answer_blocks;
    int raw_answer_size, kernel_intblock_buffer_size;
    void *raw_answer;
    INTERRUPT_BLOCK_BUFFER block;       // блок данных
    struct block_info *kernel_intblock_buffer;
    unsigned int count_blocks, i;

    int ret_code;

    ret_code = copy_from_user((void *) &block, (void *) arg, sizeof(INTERRUPT_BLOCK_BUFFER)); // получаем данные из юзерспейс
    if(ret_code)
        return ret_code;

    LOG_INFO("ioctl_read_interrupt_blocks - cnt blocks = %d", block.count_blocks);

    raw_answer_size = block.count_blocks * sizeof(meu_intblock_info);
    raw_answer = mil_kzalloc(raw_answer_size, GFP_KERNEL);
    if(raw_answer == NULL)
        return -ENOMEM;
    answer = (meu_read_reg_answer *) raw_answer;
    answer_blocks = (meu_intblock_info *)( sizeof(meu_read_reg_answer) + (char *) raw_answer);

    init_meu_header(&request.header, cfd, MEU_IOCTL_READ_BC_RAM_REQ, sizeof(request));

    ret_code = chan_simple_send(cfd, (void *) &request, MEU_IOCTL_GET_COUNT_INTERRUPT_BLOCKS_ANS, raw_answer, raw_answer_size);
    if(ret_code == 0)
    {
        ret_code = answer->ret;
        if(ret_code == 0)
        {
            count_blocks = answer->data;

            if(count_blocks > request.n_blocks)
            {
                LOG_ERROR("block count mismatch");
                count_blocks = request.n_blocks;
            }

            if(count_blocks > 0)
            {
                kernel_intblock_buffer_size = request.n_blocks * sizeof(struct block_info);
                kernel_intblock_buffer = (struct block_info *) mil_kzalloc(kernel_intblock_buffer_size, GFP_KERNEL);

                for(i = 0; i < count_blocks; ++i)
                {
                    kernel_intblock_buffer[i].free_timer_value = answer_blocks[i].free_timer_value;
                    kernel_intblock_buffer[i].interrupt_ch = answer_blocks[i].interrupt_ch;
                    kernel_intblock_buffer[i].hw_stat_reg1 = answer_blocks[i].hw_stat_reg1;
                    kernel_intblock_buffer[i].hw_stat_reg2 = answer_blocks[i].hw_stat_reg2;
                }

                ret_code = copy_to_user((void*)block.blocks, kernel_intblock_buffer, kernel_intblock_buffer_size);
                mil_kfree(kernel_intblock_buffer);
            }

            if(ret_code == 0)
                ret_code = copy_to_user((void*)arg, (void*)&count_blocks, sizeof(unsigned int));  // копируем в юзерспей фактическое кол-во блоков полученных
        }
    }

    mil_kfree(raw_answer);
    return ret_code;
}

long eth_ioctl_get_rcv_subadr_data(Mil1553ChannelFD cfd, void *arg)
{
    SUBA_DATA_BLOCK block;             // данные из юзерспейс
    meu_read_rcv_subaddr_request request;
    meu_read_rcv_subaddr_answer answer;
    int ret_code;

    LOG_INFO("eth_ioctl_get_rcv_subadr_data");
    ret_code = copy_from_user((void *) &block, (void *) arg, sizeof(block)); // получаем данные из юзерспейс
    if(ret_code)
        return ret_code;

    init_meu_header(&request.header, cfd, MEU_IOCTL_GET_RCV_SUBADR_DATA_REQ, sizeof(request));
    request.sa = block.sa;

    ret_code = chan_simple_send(cfd, (void *) &request, MEU_IOCTL_GET_RCV_SUBADR_DATA_ANS, (void *) &answer, sizeof(answer));

    if(ret_code == 0)
    {
        ret_code = answer.ret;
        if(ret_code == 0)
        {
            memcpy(block.words, answer.data_words, MEU_RCV_SUBADDR_DATA_WORDS_COUNT * sizeof(uint16_t));

            ret_code = copy_to_user((void *) arg, (void *)(&block), sizeof(block)); // кладём данные в юзерспейс
        }
    }

    return ret_code;
}

//long ioctl_set_signal_irq(Mil1553ChannelFD cfd, void *arg)
//{
//    SIGNAL_INFO info;       // инфо о сигнале
//    LOG_INFO("ioctl_set_signal_irq");
//    copy_from_user((void *) &info, (void *) arg, sizeof(SIGNAL_INFO)); // получаем данные из юзерспейс
//    if ((dev->numCh == CH_NUM_1) || (dev->numCh == CH_NUM_2) || (dev->numCh == CH_NUM_3) || (dev->numCh == CH_NUM_4)) {
//        if ((info.sig_number == SIGUSR1) || (info.sig_number == SIGUSR2)) {
//            LOG_INFO("sig_number - good");
//        } else {
//            LOG_ERROR("bad argument - info.sig_number");
//            return -EFAULT;
//        }
//        mutex_lock(&dev->mtxCh);
//        dev->sig_info.sig_num = info.sig_number;
//        dev->sig_info.target_pid = info.target_pid;
//        dev->sig_info.sig_task = pid_task( find_vpid(dev->sig_info.target_pid), PIDTYPE_PID );
//        dev->sig_info.isCanSend = 1;
//        mutex_unlock(&dev->mtxCh);
//        return 0;
//    } else {
//        LOG_ERROR("bad channel");
//        return -EFAULT;
//    }
//}

//long ioctl_clr_signal_irq(Mil1553ChannelFD cfd, void *arg)
//{
//    LOG_INFO("ioctl_clr_signal_irq");
//    if ((dev->numCh == CH_NUM_1) || (dev->numCh == CH_NUM_2) || (dev->numCh == CH_NUM_3) || (dev->numCh == CH_NUM_4)) {
//        mutex_lock(&dev->mtxCh);
//        dev->sig_info.sig_num = 0;
//        dev->sig_info.target_pid = 0;
//        dev->sig_info.sig_task = 0;
//        dev->sig_info.isCanSend = 0;
//        mutex_unlock(&dev->mtxCh);
//        return 0;
//    } else {
//        LOG_ERROR("bad channel");
//        return -EFAULT;
//    }
//}

#if defined(_WIN32) || defined(_WIN64)
# if defined(_WIN64)
typedef __int64 LONG_PTR;
# else
typedef long LONG_PTR;
# endif
typedef LONG_PTR SSIZE_T;
typedef SSIZE_T ssize_t;
#endif

ssize_t do_recv(
#ifdef _WIN32
        SOCKET
#else
        int
#endif
        socketFd, void *buf, size_t buf_size)
{
    ssize_t res;    
    ssize_t remaining_size;
    ssize_t processedSize = 0;

    for(remaining_size = buf_size; remaining_size > 0; remaining_size -= res, processedSize += res)
    {
        res = recv(socketFd, processedSize + (char *) buf, remaining_size, MSG_NOSIGNAL);
        if(res <= 0)
        {
#ifdef _WIN32
            int lastError = WSAGetLastError();
            switch(lastError)
            {
            case WSAETIMEDOUT:
            {
                res = 0;
                continue;
            }
                break;
                // The connection has been dropped because of a network failure or because the peer system failed to respond.
            case WSANOTINITIALISED:
                // A successful WSAStartup must occur before using this function.
            case WSAENETDOWN:
                // The network subsystem has failed.
            case WSAEFAULT:
                // The buf argument is not totally contained in a valid part of the user address space.
            case WSAENOTCONN:
                // The socket is not connected.
            case WSAEINTR:
                // The (blocking) call was canceled through WSACancelBlockingCall.
            case WSAEINPROGRESS:
                // A blocking Windows Sockets 1.1 call is in progress, or the service provider is still processing a callback function.
            case WSAENETRESET:
                // The connection has been broken due to the remote host resetting.
            case WSAENOTSOCK:
                // The descriptor is not a socket.
            case WSAEOPNOTSUPP:
                // MSG_OOB was specified, but the socket is not stream style such as type SOCK_STREAM, out-of-band data is not supported in the communication domain associated with this socket, or the socket is unidirectional and supports only send operations.
            case WSAESHUTDOWN:
                // The socket has been shut down; it is not possible to recv on a socket after shutdown has been invoked with how set to SD_RECEIVE or SD_BOTH.
            case WSAEWOULDBLOCK:
                // The socket is marked as nonblocking and the receive operation would block.
            case WSAEMSGSIZE:
                // The message was too large to fit into the specified buffer and was truncated.
            case WSAEINVAL:
                // The socket has not been bound with bind, or an unknown flag was specified, or MSG_OOB was specified for a socket with SO_OOBINLINE enabled or (for byte stream sockets only) len was zero or negative.
            case WSAECONNABORTED:
                // The virtual circuit was terminated due to a time-out or other failure. The application should close the socket as it is no longer usable.
            case WSAECONNRESET:
                // The virtual circuit was reset by the remote side executing a "hard" or "abortive" close. The application should close the socket as it is no longer usable. On a UDP datagram socket this error would indicate that a previous send operation resulted in an ICMP "Port Unreachable" message.
            default: break;
            }

/*          wchar_t *s = NULL;
            FormatMessageW(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
                           NULL, lastError,
                           MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                           (LPWSTR)&s, 0, NULL);
            printf("%S\n", s);
            LocalFree(s);
*/
#endif
            return (res < 0) ? res : -EINVAL;
        }
    }

    return buf_size;
}


int socket_io_fixed(Mil1553ChannelFD cfd, buffer_struct request, int op, buffer_struct response)
{
    void *in, *out, *user_buf;
    int out_size, buf_size;

    int res, in_size, rem_size, err_code;

    meu_header *out_header;

    in = request.kernel_buffer;
    out = response.kernel_buffer;
    user_buf = response.user_buffer;

    out_size = response.kernel_buffer_size;
    buf_size = response.user_buffer_size;

    out_header = (meu_header *) out;

    if (out == NULL || out_size == 0)
        return -ENOSPC;

    if (request.user_buffer != NULL)
        return -EOPNOTSUPP;

    if(in == NULL)
    {
        LOG_INFO("client:send skipped!\n");
    }
    else
    {
        in_size = ((meu_header *)in)->size;
        if(((meu_header *)in)->magic != MEU_MAGIC)
        {
            LOG_INFO("sock_sendmsg() NO MAGIC!\n");
            return -EINVAL;
        }

        res = send(cfd.socketFD, in, in_size, MSG_NOSIGNAL);

        if (res < 0)
        {
            LOG_INFO("sock_sendmsg() failed: %d\n", res);
            return res;
        }

        if (res != in_size)
        {
            LOG_INFO("sock_sendmsg() failed, cannot write full message! (%d of %d)\n", res, in_size);
            return -EINVAL;
        }

        LOG_INFO("client:send ok! (%d bytes, op %d)\n", res, ((meu_header *)in)->op);
    }

    // Читаем заголовок

    res = do_recv(cfd.socketFD, out, sizeof(meu_header));

    if (res < (int) sizeof(meu_header))
    {
        LOG_INFO("sock_recvmsg() failed: %d\n", res);
        if(res < 0)
            return res;
        else
            return -EINVAL;
    }
    else
    {
        if(out_header->magic != MEU_MAGIC)
        {
            LOG_INFO("sock_recvmsg() NO MAGIC!\n");
            return -EINVAL;
        }

        if(out_header->op != op)
        {
            LOG_INFO("sock_recvmsg() op mismatch!\n");
            return -EINVAL;
        }

        if(out_header->size > out_size && user_buf == NULL)
        {
            LOG_INFO("sock_recvmsg() size exceeds buffer!\n");
            return -EINVAL;
        }

    }

    if(out_header->size > sizeof(meu_header))
    {
        // Дочитываем остаток
        rem_size = min((int) out_header->size, out_size) - sizeof(meu_header);

        res = do_recv(cfd.socketFD, (void *)(sizeof(meu_header) + (char *)out), rem_size);

        if (res != rem_size)
        {
            LOG_INFO("sock_recvmsg() failed: %d\n", res);
            if(res < 0)
                return res;
            else
                return -EINVAL;
        }
    }

    err_code = 0;

    if(out_header->size > out_size)
    {
        if (user_buf == NULL || buf_size < (out_header->size - out_size))
        {
            LOG_INFO("sock_recvmsg() small user buffer!");
            return -ENOSPC;
        }

        buf_size = out_header->size - out_size;

        rem_size = buf_size;

        while(rem_size > 0)
        {
            int expected_size = min(rem_size, MEU_SIZE_RAW_DMA_BLOCK);

            res = do_recv(cfd.socketFD, user_buf, expected_size);

            if(res <= 0)
            {
                err_code = res;
                LOG_INFO("sock_recvmsg user failed: %d\n", err_code);
                break;
            }

            if(res != expected_size)
            {
                err_code = -EINVAL;
                LOG_INFO("sock_recvmsg user failed: %d\n", err_code);
                break;
            }

            rem_size -= res;
            user_buf = (void *)(res + (char *)user_buf);
        }
    }

    if(err_code == 0)
    {
        LOG_INFO("[fd=%d, chan=%d] receive ok! (%d bytes, op %d)\n", cfd.socketFD, cfd.channel, out_header->size, out_header->op);
    }
    else
    {
        LOG_INFO("[fd=%d, chan=%d] receive ERROR %d! (%d bytes, op %d)\n", cfd.socketFD, cfd.channel, err_code, out_header->size, out_header->op);
    }

    return err_code;
}


int chan_simple_send(Mil1553ChannelFD cfd, void *request, int op, void *answer, size_t answer_size)
{
    buffer_struct buf_request;
    buffer_struct buf_response;

    memset(&buf_request, 0, sizeof(buffer_struct));
    memset(&buf_response, 0, sizeof(buffer_struct));

    if (cfd.socketFD <= 0 || answer == NULL || request == NULL)
    {
        LOG_INFO("chan_simple_send NULLPTR!\n");
        return -ENODEV;
    }

    if(((meu_header *)request)->magic != MEU_MAGIC)
    {
        LOG_INFO("chan_simple_send NO MAGIC!\n");
        return -EINVAL;
    }

    buf_request.kernel_buffer = request;
    buf_request.kernel_buffer_size = ((meu_header *)request)->size;

    buf_response.kernel_buffer = answer;
    buf_response.kernel_buffer_size = answer_size;

    return chan_send_with_userbuf(cfd, buf_request, op, buf_response);
}

int chan_send_with_userbuf(Mil1553ChannelFD cfd, buffer_struct request, int op, buffer_struct response)
{
    int err_code = -ENODEV;
    if (cfd.socketFD <= 0 || request.kernel_buffer == NULL || response.kernel_buffer == NULL)
    {
        LOG_INFO("[fd=%d, chan=%d] chan_send_with_userbuf NULLPTR!\n", cfd.socketFD, cfd.channel);
        return -ENODEV;
    }

    err_code = socket_io_fixed(cfd, request, op, response);

    if (err_code != 0)
    {
        LOG_INFO("[fd=%d, chan=%d] error %d\n", cfd.socketFD, cfd.channel, err_code);
    }

    return err_code;
}
