/// \file extmil1553ud_extioctl_wrappers.h
/// \brief библиотка взаимодействия - обёртки ioctl
/// заголовки функций
#ifndef EXTMIL1553UD_IOCTL_WRAPPERS_H
#define EXTMIL1553UD_IOCTL_WRAPPERS_H

#include "mil1553udext_library.h"
#include "mil1553ud_cmd.h"
#include "mil1553ud_regs.h"

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <time.h>
#include <errno.h>
#include <ctype.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#ifdef _WIN32
  #include <winsock2.h>
  #include <windows.h>
#else
  #include <unistd.h>
  #include <strings.h>
  #include <sys/time.h>
  #include <dirent.h>
  #include <sys/param.h>
  #include <sys/ioctl.h>  /* ioctl */
#endif

#ifdef _WIN32
    #include <winsock2.h>
    #include <windows.h>

/* ioctl command encoding: 32 bits total, command in lower 16 bits,
 * size of the parameter structure in the lower 14 bits of the
 * upper 16 bits.
 * Encoding the size of the parameter structure in the ioctl request
 * is useful for catching programs compiled with old versions
 * and to avoid overwriting user space outside the user buffer area.
 * The highest 2 bits are reserved for indicating the ``access mode''.
 * NOTE: This limits the max parameter size to 16kB -1 !
 */

/*
 * The following is for compatibility across the various Linux
 * platforms.  The generic ioctl numbering scheme doesn't really enforce
 * a type field.  De facto, however, the top 8 bits of the lower 16
 * bits are indeed used as a type field, so we might just as well make
 * this explicit here.  Please be sure to use the decoding macros
 * below from now on.
 */
#define _IOC_NRBITS	8
#define _IOC_TYPEBITS	8

/*
 * Let any architecture override either of the following before
 * including this file.
 */

#ifndef _IOC_SIZEBITS
# define _IOC_SIZEBITS	14
#endif

#ifndef _IOC_DIRBITS
# define _IOC_DIRBITS	2
#endif

#define _IOC_NRMASK	((1 << _IOC_NRBITS)-1)
#define _IOC_TYPEMASK	((1 << _IOC_TYPEBITS)-1)
#define _IOC_SIZEMASK	((1 << _IOC_SIZEBITS)-1)
#define _IOC_DIRMASK	((1 << _IOC_DIRBITS)-1)

#define _IOC_NRSHIFT	0
#define _IOC_TYPESHIFT	(_IOC_NRSHIFT+_IOC_NRBITS)
#define _IOC_SIZESHIFT	(_IOC_TYPESHIFT+_IOC_TYPEBITS)
#define _IOC_DIRSHIFT	(_IOC_SIZESHIFT+_IOC_SIZEBITS)

/*
 * Direction bits, which any architecture can choose to override
 * before including this file.
 *
 * NOTE: _IOC_WRITE means userland is writing and kernel is
 * reading. _IOC_READ means userland is reading and kernel is writing.
 */

#ifndef _IOC_NONE
# define _IOC_NONE	0U
#endif

#ifndef _IOC_WRITE
# define _IOC_WRITE	1U
#endif

#ifndef _IOC_READ
# define _IOC_READ	2U
#endif

#define _IOC(dir,type,nr,size) \
    (((dir)  << _IOC_DIRSHIFT) | \
     ((type) << _IOC_TYPESHIFT) | \
     ((nr)   << _IOC_NRSHIFT) | \
     ((size) << _IOC_SIZESHIFT))

#ifndef __KERNEL__
#define _IOC_TYPECHECK(t) (sizeof(t))
#endif

/*
 * Used to create numbers.
 *
 * NOTE: _IOW means userland is writing and kernel is reading. _IOR
 * means userland is reading and kernel is writing.
 */

#ifndef _IO
    #define _IO(type,nr)		_IOC(_IOC_NONE,(type),(nr),0)
#endif

#ifndef _IOR
    #define _IOR(type,nr,size)	_IOC(_IOC_READ,(type),(nr),(_IOC_TYPECHECK(size)))
#endif

#ifndef _IOW
    #define _IOW(type,nr,size)	_IOC(_IOC_WRITE,(type),(nr),(_IOC_TYPECHECK(size)))
#endif

#define _IOWR(type,nr,size)	_IOC(_IOC_READ|_IOC_WRITE,(type),(nr),(_IOC_TYPECHECK(size)))
#define _IOR_BAD(type,nr,size)	_IOC(_IOC_READ,(type),(nr),sizeof(size))
#define _IOW_BAD(type,nr,size)	_IOC(_IOC_WRITE,(type),(nr),sizeof(size))
#define _IOWR_BAD(type,nr,size)	_IOC(_IOC_READ|_IOC_WRITE,(type),(nr),sizeof(size))

/* used to decode ioctl numbers.. */
#define _IOC_DIR(nr)		(((nr) >> _IOC_DIRSHIFT) & _IOC_DIRMASK)
#define _IOC_TYPE(nr)		(((nr) >> _IOC_TYPESHIFT) & _IOC_TYPEMASK)
#define _IOC_NR(nr)		(((nr) >> _IOC_NRSHIFT) & _IOC_NRMASK)
#define _IOC_SIZE(nr)		(((nr) >> _IOC_SIZESHIFT) & _IOC_SIZEMASK)

/* ...and for the drivers/sound files... */

#ifndef IOC_IN
    #define IOC_IN		(_IOC_WRITE << _IOC_DIRSHIFT)
#endif

#ifndef IOC_OUT
    #define IOC_OUT		(_IOC_READ << _IOC_DIRSHIFT)
#endif

#ifndef IOC_INOUT
    #define IOC_INOUT	((_IOC_WRITE|_IOC_READ) << _IOC_DIRSHIFT)
#endif

#define IOCSIZE_MASK	(_IOC_SIZEMASK << _IOC_SIZESHIFT)
#define IOCSIZE_SHIFT	(_IOC_SIZESHIFT)

#endif

/// \remark ioctl-команды драйвера

#define IOC_MAGIC 'b'
// read/write regs
/// \brief команда "запись в регистр"
/// позволяет записть значение в заданный регистр управления
#define IOCTL_WRITE_REG                             _IOW(IOC_MAGIC, 0,  SADDR_DATA)             // запись в регистр
/// \brief команда "запись в регистр заданных бит"
/// позволяет изменить отдельные биты в заданном регистре управления
#define IOCTL_WRITE_REG_BIT_MASK                    _IOW(IOC_MAGIC, 1,  SADDR_DATA_BIT_MASK)    // запись в регистр заданных бит
/// \brief команда "чтение из регистра"
/// позволяет прочитать значение из заданного регистра управления
#define IOCTL_READ_REG                              _IOWR(IOC_MAGIC,2,  SADDR_DATA)             // чтение из регитсра
// info command
/// \brief команда "чтение подробной информации о плате и драйвере"
/// позволяет считать подробную информацию о pci-плате
#define IOCTL_VERSION                               _IOR(IOC_MAGIC, 31, VERSION)                // получить информацию  о плате
/// \brief команда "чтение версии драйвера"
/// позволяет считать версию и дату драйвера
#define IOCTL_VERSION_DRIVER                        _IOR(IOC_MAGIC, 40, unsigned int)           // получить информацию о драйвере
/// \brief команда "чтение информации о pci-локации платы"
/// позволяет считать информацию pci
#define IOCTL_PCI_LOCATION                          _IOR(IOC_MAGIC, 41, PCI_LOCATION)           // получить информацию о pci-локации платы

/// \brief команда "чтение информации о ethernet-локации платы"
/// позволяет считать IP-адрес и порт
#define IOCTL_ETH_LOCATION                          _IOR(IOC_MAGIC, 42, ETH_LOCATION)           // получить информацию о ethernet-локации платы

/// \brief команда "чтение информации о usb-локации платы"
/// позволяет считать путь устройства
#define IOCTL_USB_LOCATION                          _IOR(IOC_MAGIC, 43, USB_LOCATION)           // получить путь устройства

// dma
/// \brief команда "сброс указателя dma канала"
/// позволяет обнулить DMA_INDEX и программные указатели чтения/записи
#define IOCTL_CLEAR_DMA                             _IO(IOC_MAGIC,  5)                          // сброс указателя dma канала
/// \brief команда "включить dma устройства"
/// разрешает работу DMA
#define IOCTL_ENABLE_DMA                            _IO(IOC_MAGIC,  8)                          // включить dma платы
/// \brief команда "выключить dma устройства"
/// запрещает работу DMA
#define IOCTL_DISABLE_DMA                           _IO(IOC_MAGIC,  9)                          // выключить dma платы
/// \brief команда "получить количество 128 байтных блоков из ДМА"
/// позволяет получить количество готовых для чтения блоков из буфера ДМА
#define IOCTL_GET_NBLOCK_RAW_DMA                    _IOWR(IOC_MAGIC,7,  unsigned int)           // получить количество блоков из ДМА в сыром виде
/// \brief команда "считать заданное количество 128 байтных из ДМА"
/// позволяет получить данные (блоки) из буфера ДМА
#define IOCTL_READ_BLOCKS_RAW_DMA                   _IOWR(IOC_MAGIC,6,  DMA_READ_BLOCK)         // считать заданное кол-во блоков из ДМА
#define IOCTL_READ_BLOCKS_RAW_DMA_FIX_COUNT         _IOWR(IOC_MAGIC,60, MIL1553_DMA_FIX_BLOCK)      // считать фиксированное кол-во блоков из ДМА (не более 256 за раз)
// irq
/// \brief команда "управление прерываниями INTERRUPT_MASK"
#define IOCTL_MANAGE_INTERRUPT                      _IOW(IOC_MAGIC, 16, INTERRUPT_MAN)          // управление прерываниями INTERRUPT_MASK
/// \brief команда "управление прерываниями по передаче из подадреса"
#define IOCTL_MAN_IRQ_SUBADDRESS_TR                 _IOW(IOC_MAGIC, 38, INTERRUPT_SUBADDRESS)   // управление прерываниями по передаче из подадреса
/// \brief команда "управление прерываниями по приёму в подадрес"
#define IOCTL_MAN_IRQ_SUBADDRESS_RCV                _IOW(IOC_MAGIC, 39, INTERRUPT_SUBADDRESS)   // управление прерываниями по приёму в подадрес
// mode
/// \brief команда "выбор режима работы канала"
#define IOCTL_SWITCH_MODE                           _IOW(IOC_MAGIC, 4,  unsigned int)           // выбор режима работы канала
/// \brief команда "выбор шины канала"
#define IOCTL_SWITCH_BUS                            _IOW(IOC_MAGIC, 33, unsigned int)           // выбор шины канала
/// \brief команда "разрешить работу канала"
#define IOCTL_DEV_CHANNEL_WORK                      _IOW(IOC_MAGIC, 34, unsigned int)           // вкл/выкл канал
// term dev
/// \brief команда "установить режим ОУ"
#define IOCTL_SET_RT_TR_MODE                        _IOW(IOC_MAGIC, 35, RT_TR_MODE)             // установить режим ОУ
/// \brief команда "установить готовность буфера ОУ"
#define IOCTL_SET_RT_TR_BUF_READY                   _IOW(IOC_MAGIC, 37, RT_TR_MODE_BUF)         // установить готовность буфера ОУ
/// \brief команда "управление разрешением буферов"
#define IOCTL_RT_BUF_MAN_EN                         _IOW(IOC_MAGIC, 36, RT_BUF_MAN_EN)          // установка разрешение буферов ОУ
// channel
/// \brief команда "установить адрес ОУ"
#define IOCTL_SET_ADDRESS                           _IOW(IOC_MAGIC, 19, unsigned char)          // установить адрес ОУ
/// \brief команда "установить таймауты ОУ"
#define IOCTL_SET_TIMER_RTBM_CONF_REG_PCI           _IOW(IOC_MAGIC, 20, TIMER_TIMEOUT)          // set timer and timeout
/// \brief команда "установить таймауты КШ"
#define IOCTL_SET_TIMER_BC_CONF_REG_PCI             _IOW(IOC_MAGIC, 32, TIMER_TIMEOUT)          // set timer and timeout
// paddr and buf
/// \brief команда "записать подадрес на отправку"
#define IOCTL_WR_BLOCK_BUF_SUBADR                   _IOW(IOC_MAGIC, 27, RT_TR_BUF_SUBADDR)      // записать поддадрес
/// \brief команда "прочитать подадрес на отправку"
#define IOCTL_RD_BLOCK_BUF_SUBADR                   _IOWR(IOC_MAGIC,28, RT_TR_BUF_SUBADDR)      // прочитать поддадрес
// bc_ram
/// \brief команда "запись блока в BC RAM"
#define IOCTL_WRITE_BC_RAM                          _IOW(IOC_MAGIC, 29, BC_RAM_BLOCK)           // запись в BC ram
/// \brief команда "чтение блока из BC RAM"
#define IOCTL_READ_BC_RAM                           _IOWR(IOC_MAGIC,30, BC_RAM_BLOCK)           // чтение из BC ram
// interrupt ring buffer
/// \brief получить кол-во блоков с информацией о возникших прерываниях из кольцевого буфера
#define IOCTL_GET_COUNT_INTERRUPT_BLOCKS            _IOWR(IOC_MAGIC,50, unsigned int)           // получить кол-во блоков с информацией о возникших прерываниях
/// \brief считать заданное кол-во блоков из кольцевого буфера
#define IOCTL_READ_INTERRUPT_BLOCKS                 _IOWR(IOC_MAGIC,51, INTERRUPT_BLOCK_BUFFER) // считать заданное кол-во блоков из кольцевого буфера
/// \brief считать входной поддадрес
#define IOCTL_GET_RCV_SUBADR_DATA                   _IOWR(IOC_MAGIC,52, SUBA_DATA_BLOCK)        // считать входной поддадрес
/// \brief получить номер канала
#define IOCTL_GET_NUMBER_CH                         _IOWR(IOC_MAGIC,55,  unsigned int)          // получить номер канала
// signals
//#define IOCTL_SET_SIGNAL_IRQ                        _IOW(IOC_MAGIC, 53, SIGNAL_INFO)            // установить сигнал для прерывания
//#define IOCTL_CLR_SIGNAL_IRQ                        _IO(IOC_MAGIC,  54)



/// \brief Получить количество блоков обновлений
#define IOCTL_FW_COUNT                              _IO(IOC_MAGIC, 44)          // статус обновления

/// \brief статус обновления
#define IOCTL_FW_STATUS                             _IOWR(IOC_MAGIC, 45, FW_STATUS)          // статус обновления

/// \brief обновить ПО для lan1553ud (инициализация)
#define IOCTL_FW_BEGIN                              _IOWR(IOC_MAGIC, 46, CONF_BLOCK)           // обновить ПО (инициализация)

/// \brief обновить ПО для lan1553ud (загрузка блока)
#define IOCTL_FW_LOAD                               _IOW(IOC_MAGIC, 47, FW_BLOCK) // обновить ПО (загрузка блока)

/// \brief обновить ПО для lan1553ud (прошивка)
#define IOCTL_FW_UPDATE_APPLY                       _IO(IOC_MAGIC, 48)           // обновить ПО (прошивка)

/// \brief обновить ПО для lan1553ud (отмена прошивки)
#define IOCTL_FW_UPDATE_CANCEL                      _IO(IOC_MAGIC, 49)           // обновить ПО (отмена прошивки)

/// \brief загрузить настройки ПО lan1553ud
#define IOCTL_FW_WR_SETTINGS                       _IOWR(IOC_MAGIC, 56, CONF_BLOCK) // обновить ПО (загрузка блока)

/// \brief скачать настройки ПО lan1553ud
#define IOCTL_FW_RD_SETTINGS                       _IOWR(IOC_MAGIC, 57, CONF_BLOCK) // обновить ПО (загрузка блока)

/// \brief перезагрузить lan1553ud
#define IOCTL_EXT_REBOOT                           _IO(IOC_MAGIC, 61) // перезагрузка внешнего устройства

/// \brief получить индекс активного раздела lan1553ud
#define IOCTL_EXT_GET_CURRENT                      _IO(IOC_MAGIC, 62) // получить индекс активного раздела lan1553ud

/// \brief получить количество каналов lan1553ud
#define IOCTL_EXT_GET_COUNT_CHANNELS                _IO(IOC_MAGIC, 63) // получить количество каналов lan1553ud


#ifdef __cplusplus
extern "C" {
#endif

/// /// /// /// /// /// /// /// /// /// /// ///
/// \remark МАКРОБИБЛИОТЕКА EXTMIL1553. Заголовки отладочных IOCTL функций
/// /// /// /// /// /// /// /// /// /// /// ///

/// \brief Режим отладки библиотеки
// #define DEBUG_MIL1553_LIB

/// \remark подробное описание IOCTL вызовов находится в файле "extmil1553ud_cmd.h"

unsigned int extioctl_IOCTL_READ_BLOCKS_RAW_DMA(Mil1553ChannelFD cfd, DMA_READ_BLOCK *block);
unsigned int extioctl_IOCTL_READ_BLOCKS_RAW_DMA_FIX_BLOCKS(Mil1553ChannelFD cfd, MIL1553_DMA_FIX_BLOCK *block);
unsigned int extioctl_IOCTL_WRITE_REG(Mil1553ChannelFD cfd, SADDR_DATA *reg);
unsigned int extioctl_IOCTL_READ_REG(Mil1553ChannelFD cfd, SADDR_DATA *reg) ;
unsigned int extioctl_IOCTL_WRITE_REG_BIT_MASK(Mil1553ChannelFD cfd, SADDR_DATA_BIT_MASK *reg);
unsigned int extioctl_IOCTL_VERSION(Mil1553ChannelFD cfd, VERSION *info);
unsigned int extioctl_IOCTL_VERSION_DRIVER(Mil1553ChannelFD cfd, unsigned int *version);

unsigned int extioctl_IOCTL_GET_COUNT_CHANNELS(Mil1553ChannelFD cfd);

unsigned int extioctl_IOCTL_CLEAR_DMA(Mil1553ChannelFD cfd);
unsigned int extioctl_IOCTL_ENABLE_DMA(Mil1553ChannelFD cfd);
unsigned int extioctl_IOCTL_DISABLE_DMA(Mil1553ChannelFD cfd);
unsigned int extioctl_IOCTL_GET_NBLOCK_RAW_DMA(Mil1553ChannelFD cfd, unsigned int* countBlocks);
unsigned int extioctl_IOCTL_MANAGE_INTERRUPT(Mil1553ChannelFD cfd, INTERRUPT_MAN* man);
unsigned int extioctl_IOCTL_MAN_IRQ_SUBADDRESS_TR(Mil1553ChannelFD cfd, INTERRUPT_SUBADDRESS* man);
unsigned int extioctl_IOCTL_MAN_IRQ_SUBADDRESS_RCV(Mil1553ChannelFD cfd, INTERRUPT_SUBADDRESS* man);
unsigned int extioctl_IOCTL_SWITCH_MODE(Mil1553ChannelFD cfd, unsigned int* role);
unsigned int extioctl_IOCTL_SWITCH_BUS(Mil1553ChannelFD cfd, unsigned int* bus);
unsigned int extioctl_IOCTL_DEV_CHANNEL_WORK(Mil1553ChannelFD cfd, unsigned int* op);
unsigned int extioctl_IOCTL_SET_RT_TR_MODE(Mil1553ChannelFD cfd, RT_TR_MODE* rt_tr_mode);
unsigned int extioctl_IOCTL_SET_RT_TR_BUF_READY(Mil1553ChannelFD cfd, RT_TR_MODE_BUF* rdBuf);
unsigned int extioctl_IOCTL_RT_BUF_MAN_EN(Mil1553ChannelFD cfd, RT_BUF_MAN_EN* rtBufMan);
unsigned int extioctl_IOCTL_SET_ADDRESS(Mil1553ChannelFD cfd, unsigned char* address);
unsigned int extioctl_IOCTL_SET_TIMER_RTBM_CONF_REG_PCI(Mil1553ChannelFD cfd, TIMER_TIMEOUT* tmr);
unsigned int extioctl_IOCTL_SET_TIMER_BC_CONF_REG_PCI (Mil1553ChannelFD cfd, TIMER_TIMEOUT* tmr);
unsigned int extioctl_IOCTL_WR_BLOCK_BUF_SUBADR (Mil1553ChannelFD cfd, RT_TR_BUF_SUBADDR* buf);
unsigned int extioctl_IOCTL_RD_BLOCK_BUF_SUBADR (Mil1553ChannelFD cfd, RT_TR_BUF_SUBADDR* buf);
unsigned int extioctl_IOCTL_WRITE_BC_RAM(Mil1553ChannelFD cfd, BC_RAM_BLOCK* block);
unsigned int extioctl_IOCTL_READ_BC_RAM(Mil1553ChannelFD cfd, BC_RAM_BLOCK* block);
unsigned int extioctl_IOCTL_GET_COUNT_INTERRUPT_BLOCKS(Mil1553ChannelFD cfd, unsigned int* countBlocks);
unsigned int extioctl_IOCTL_READ_INTERRUPT_BLOCKS(Mil1553ChannelFD cfd, INTERRUPT_BLOCK_BUFFER* block);
unsigned int extioctl_IOCTL_GET_RCV_SUBADR_DATA(Mil1553ChannelFD cfd, SUBA_DATA_BLOCK* block);
unsigned int extioctl_IOCTL_GET_NUMBER_CH(Mil1553ChannelFD cfd, unsigned int* num);

/// /// /// /// /// /// /// /// /// /// /// ///
/// \remark МАКРОБИБЛИОТЕКА EXTMIL1553. ОБЁРТКИ для ВЫЗОВА IOCTL
/// /// /// /// /// /// /// /// /// /// /// ///

  #define WRAPPER_IOCTL_READ_BLOCKS_RAW_DMA(fd, block) extioctl_IOCTL_READ_BLOCKS_RAW_DMA(fd, block)
  #define WRAPPER_IOCTL_READ_BLOCKS_RAW_DMA_FIX_BLOCKS(fd, block) extioctl_IOCTL_READ_BLOCKS_RAW_DMA_FIX_BLOCKS(fd, block)
  #define WRAPPER_IOCTL_WRITE_REG(fd, reg) extioctl_IOCTL_WRITE_REG(fd, reg)
  #define WRAPPER_IOCTL_READ_REG(fd, reg) extioctl_IOCTL_READ_REG(fd, reg)
  #define WRAPPER_IOCTL_WRITE_REG_BIT_MASK(fd, reg) extioctl_IOCTL_WRITE_REG_BIT_MASK(fd, reg)
  #define WRAPPER_IOCTL_VERSION(fd, info) extioctl_IOCTL_VERSION(fd, info)
  #define WRAPPER_IOCTL_VERSION_DRIVER(fd, version) extioctl_IOCTL_VERSION_DRIVER(fd, version)
  #define WRAPPER_IOCTL_PCI_LOCATION(fd, pciInfo) extioctl_IOCTL_PCI_LOCATION(fd, pciInfo)
  #define WRAPPER_IOCTL_ETH_LOCATION(fd, ethInfo) extioctl_IOCTL_ETH_LOCATION(fd, ethInfo)
  #define WRAPPER_IOCTL_USB_LOCATION(fd, usbInfo) extioctl_IOCTL_USB_LOCATION(fd, usbInfo)
  #define WRAPPER_IOCTL_CLEAR_DMA(fd) extioctl_IOCTL_CLEAR_DMA(fd)
  #define WRAPPER_IOCTL_ENABLE_DMA(fd) extioctl_IOCTL_ENABLE_DMA(fd)
  #define WRAPPER_IOCTL_DISABLE_DMA(fd) extioctl_IOCTL_DISABLE_DMA(fd)
  #define WRAPPER_IOCTL_GET_NBLOCK_RAW_DMA(fd, countBlocks) extioctl_IOCTL_GET_NBLOCK_RAW_DMA(fd, countBlocks)
  #define WRAPPER_IOCTL_MANAGE_INTERRUPT(fd, man) extioctl_IOCTL_MANAGE_INTERRUPT(fd, man)
  #define WRAPPER_IOCTL_MAN_IRQ_SUBADDRESS_TR(fd, man) extioctl_IOCTL_MAN_IRQ_SUBADDRESS_TR(fd, man)
  #define WRAPPER_IOCTL_MAN_IRQ_SUBADDRESS_RCV(fd, man) extioctl_IOCTL_MAN_IRQ_SUBADDRESS_RCV(fd, man)
  #define WRAPPER_IOCTL_GET_COUNT_CHANNELS(fd) extioctl_IOCTL_GET_COUNT_CHANNELS(fd)
  #define WRAPPER_IOCTL_SWITCH_MODE(fd, role) extioctl_IOCTL_SWITCH_MODE(fd, role)
  #define WRAPPER_IOCTL_SWITCH_BUS(fd, bus) extioctl_IOCTL_SWITCH_BUS(fd, bus)
  #define WRAPPER_IOCTL_DEV_CHANNEL_WORK(fd, op) extioctl_IOCTL_DEV_CHANNEL_WORK(fd, op)
  #define WRAPPER_IOCTL_SET_RT_TR_MODE(fd, rt_tr_mode) extioctl_IOCTL_SET_RT_TR_MODE(fd, rt_tr_mode)
  #define WRAPPER_IOCTL_SET_RT_TR_BUF_READY(fd, rdBuf) extioctl_IOCTL_SET_RT_TR_BUF_READY(fd, rdBuf)
  #define WRAPPER_IOCTL_RT_BUF_MAN_EN(fd, rtBufMan) extioctl_IOCTL_RT_BUF_MAN_EN(fd, rtBufMan)
  #define WRAPPER_IOCTL_SET_ADDRESS(fd, address) extioctl_IOCTL_SET_ADDRESS(fd, address)
  #define WRAPPER_IOCTL_SET_TIMER_RTBM_CONF_REG_PCI(fd, tmr) extioctl_IOCTL_SET_TIMER_RTBM_CONF_REG_PCI(fd, tmr)
  #define WRAPPER_IOCTL_SET_TIMER_BC_CONF_REG_PCI(fd, tmr) extioctl_IOCTL_SET_TIMER_BC_CONF_REG_PCI(fd, tmr)
  #define WRAPPER_IOCTL_WR_BLOCK_BUF_SUBADR(fd, buf) extioctl_IOCTL_WR_BLOCK_BUF_SUBADR(fd, buf)
  #define WRAPPER_IOCTL_RD_BLOCK_BUF_SUBADR(fd, buf) extioctl_IOCTL_RD_BLOCK_BUF_SUBADR(fd, buf)
  #define WRAPPER_IOCTL_WRITE_BC_RAM(fd, block) extioctl_IOCTL_WRITE_BC_RAM(fd, block)
  #define WRAPPER_IOCTL_READ_BC_RAM(fd, block) extioctl_IOCTL_READ_BC_RAM(fd, block)
  #define WRAPPER_IOCTL_GET_COUNT_INTERRUPT_BLOCKS(fd, countBlocks) extioctl_IOCTL_GET_COUNT_INTERRUPT_BLOCKS(fd, countBlocks)
  #define WRAPPER_IOCTL_READ_INTERRUPT_BLOCKS(fd, block) extioctl_IOCTL_READ_INTERRUPT_BLOCKS(fd, block)
  #define WRAPPER_IOCTL_GET_RCV_SUBADR_DATA(fd, block) extioctl_IOCTL_GET_RCV_SUBADR_DATA(fd, block)
  #define WRAPPER_IOCTL_GET_NUMBER_CH(fd, num) extioctl_IOCTL_GET_NUMBER_CH(fd, num)

#ifdef __cplusplus
}
#endif

#endif // EXTMIL1553UD_IOCTL_WRAPPERS_H
