/// \file extil1553ud_library.h
/// \brief библиотка взаимодействия
/// заголовки функций
#ifndef EXTMIL1553UD_LIBRARY_H
#define EXTMIL1553UD_LIBRARY_H

#include "mil1553ud_common.h"

#if !defined _WIN32 || defined ENABLE_LINKING_FROM_SOURCES
#define MIL1553UDEXT_API
#endif

#if defined _WIN32 && !defined ENABLE_LINKING_FROM_SOURCES

#ifdef LIB_MIL1553UDEXT
#define MIL1553UDEXT_API __declspec(dllexport)
#else
#define MIL1553UDEXT_API __declspec(dllimport)
#endif

#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _WIN32

///
/// \brief Дескриптор канала внешнего устройства
///
typedef struct
{
    SOCKET socketFD; ///< дескриптор сокета
    unsigned int channel; ///< канал внешнего устройства
} Mil1553ChannelFD;

#else

///
/// \brief Дескриптор канала внешнего устройства
///
typedef struct
{
    int socketFD; ///< дескриптор сокета
    unsigned int channel; ///< канал внешнего устройства
} Mil1553ChannelFD;

#endif


/// \remark Старые функции имеют префикс mil1553ud_
/// Новые функции имеют префикс mil1553_

/// /// /// /// /// /// /// /// /// /// /// ///
/// \remark МАКРОБИБЛИОТЕКА MIL1553. ТИПЫ ДАННЫХ и ОБЪЯВЛЕНИЯ.
/// /// /// /// /// /// /// /// /// /// /// ///

/// /// /// /// /// /// /// /// /// /// /// ///
/// \remark МАКРОБИБЛИОТЕКА MIL1553. ФУНКЦИИ.
/// /// /// /// /// /// /// /// /// /// /// ///

/// \brief подключиться к каналу связи
/// \param ipv4_addr адрес устройства (строка, например "192.168.10.10")
/// \param port порт устройства
/// \param channel mil-канал устройства
/// \return дескриптор открытого канала (проверьте, что socketFd не -1)
MIL1553UDEXT_API Mil1553ChannelFD extmil1553_open(const char *address, uint16_t port, uint16_t channel);

/// \brief закрыть канал связи
MIL1553UDEXT_API int extmil1553_close(Mil1553ChannelFD cfd);

///
/// \brief Чтение данных (блоков) из ДМА буфера канала
/// \param cfd дескриптор канала
/// \param container контейнер блоков ДМА
/// \return результат операции
///
MIL1553UDEXT_API unsigned int extmil1553_readDma(Mil1553ChannelFD cfd, struct MIL1553_DMA_CONTAINER_BLOCKS* container);

///
/// \brief чтение фиксированное кол-во блоков
/// \param cfd дескриптор канала
/// \param container контейнер блоков ДМА
/// \return результат операции
///
MIL1553UDEXT_API unsigned int extmil1553_readDmaFixBlocks(Mil1553ChannelFD cfd, MIL1553_DMA_FIX_BLOCK* container);

///
/// \brief Вкл./выкл. канала
/// \param cfd дескриптор канала
/// \param op MIL_DEV_CHANNEL_ON, MIL_DEV_CHANNEL_OFF
/// \return результат операции
///
MIL1553UDEXT_API unsigned int extmil1553_manChannel(Mil1553ChannelFD cfd, unsigned int op);

///
/// \brief Выбор роли (режима) канала КШ, ОУ, МШ, ОУМШ
/// \param cfd дескриптор канала
/// \param role роль (режим)
/// \return результат операции
///
MIL1553UDEXT_API unsigned int extmil1553_setRole(Mil1553ChannelFD cfd, unsigned int role);

///
/// \brief получить кол-во каналов внешнего устройства
/// \param cfd дескриптор канала
/// \return кол-во каналов
///
MIL1553UDEXT_API int extmil1553_getCountChannels(Mil1553ChannelFD cfd);

///
/// \brief записать значение регистра
/// \param cfd дескриптор канала
/// \param reg инфо о регистре
/// \return результат операции
///
MIL1553UDEXT_API unsigned int extmil1553_writeReg(Mil1553ChannelFD cfd, SADDR_DATA* reg);

///
/// \brief прочитать значение регистра
/// \param cfd дескриптор канала
/// \param reg инфо о регистре
/// \return результат операции
///
MIL1553UDEXT_API unsigned int extmil1553_readReg(Mil1553ChannelFD cfd, SADDR_DATA* reg);

///
/// \brief изменить заданные биты регистра
/// \param cfd дескриптор канала
/// \param reg инфо о регистре
/// \return результат операции
///
MIL1553UDEXT_API unsigned int extmil1553_modifyReg(Mil1553ChannelFD cfd, SADDR_DATA_BIT_MASK* reg);

///
/// \brief получить информацию о канале и плате
/// \param cfd дескриптор канала
/// \param info инфо о канале и плате
/// \return результат операции
///
MIL1553UDEXT_API unsigned int extmil1553_getDeviceInfo(Mil1553ChannelFD cfd, VERSION* info);

///
/// \brief получить версию драйверу
/// \param cfd дескриптор канала
/// \param version версия драйвера
/// \return результат операции
///
MIL1553UDEXT_API unsigned int extmil1553_getDriverVersion(Mil1553ChannelFD cfd, unsigned int* version);

///
/// \brief Сброс ДМА
/// \param cfd дескриптор канала
/// \return результат операции
///
MIL1553UDEXT_API unsigned int extmil1553_clearDma(Mil1553ChannelFD cfd);

///
/// \brief управление ДМА
/// \param cfd дескриптор канала
/// \param op 0 - выкл, !0 - вкл
/// \return результат операции
///
MIL1553UDEXT_API unsigned int extmil1553_manDma(Mil1553ChannelFD cfd, int op);

///
/// \brief Выбор шины А и/или Б
/// \param cfd дескриптор канала
/// \param bus шина - MIL_BUS_A_EN, MIL_BUS_B_EN
/// \return результат операции
///
MIL1553UDEXT_API unsigned int extmil1553_switchBus(Mil1553ChannelFD cfd, unsigned int bus);

///
/// \brief запись данных в RAM контроллера шины
/// \param cfd дескриптор канала
/// \param type тип озу контроллера шины
/// \param data данные (массив)
/// \param length размер массива
/// \return результат операции
///
MIL1553UDEXT_API unsigned int extmil1553_bc_writeDataToRam(Mil1553ChannelFD cfd, enum EMil1553BcTypeDataRam type, unsigned int* data, int length);

///
/// \brief Разрешение подадресов ОУ на приём/передачу
/// \param cfd дескриптор канала
/// \param direction направление MIL_RT_BUF_RECEIVE, MIL_RT_BUF_TRANSMIT
/// \param subadr_mask маска подадресов (1 - изменять подадрес, 0 - не изменять поадрес) (с 1 по 30 подадрес)
/// \param action_mask маска действий (1 - разрешить, 0 - запретить) (с 1 по 30 подадрес)
/// \return результат операции
///
MIL1553UDEXT_API unsigned int extmil1553_rt_manSubaddresses(Mil1553ChannelFD cfd, unsigned int direction, unsigned int subadr_mask, unsigned int action_mask);

///
/// \brief установить адрес ОУ
/// \param cfd дескриптор канала
/// \param address адрес ОУ
/// \return результат операции
///
MIL1553UDEXT_API unsigned int extmil1553_rt_setAddress(Mil1553ChannelFD cfd, unsigned char address);

///
/// \brief управление режимом буфера ОУ на передачу
/// \param cfd дескриптор канала
/// \param subaddress подадрес
/// \param mode режим буфера MIL_RT_MODE_PROG, MIL_RT_MODE_HRDW
/// \return результат операции
///
MIL1553UDEXT_API unsigned int extmil1553_rt_manSubadressTransmitBufferMode(Mil1553ChannelFD cfd, unsigned char subaddress, unsigned int mode);

///
/// \brief управление готовностью буфера ОУ на передачу
/// \param cfd дескриптор канала
/// \param subaddress подадрес
/// \param cmd_buf команды управления буферами
/// \return результат операции
///
MIL1553UDEXT_API unsigned int extmil1553_rt_manSubaddressTransmitBufferReady(Mil1553ChannelFD cfd, unsigned char subaddress, unsigned int cmd_buf);

///
/// \brief запись данных в подадрес отправного буфера ОУ
/// \param cfd дескриптор канала
/// \param num_buf номер аппаратного буфера MIL_RT_BUF0, MIL_RT_BUF1
/// \param subaddress подадрес
/// \param data данные
/// \return результат операции
///
MIL1553UDEXT_API unsigned int extmil1553_rt_writeDataToSubaddressTransmit(Mil1553ChannelFD cfd, unsigned int num_buf, unsigned char subaddress, struct TRtSubaddressData data);

/*-------------------------------------------------------------------------------------------------------*/

/// /// /// /// /// /// /// /// /// ///
/// \remark ОБЩИЕ СЕРВИСНЫЕ ФУНКЦИИ
/// /// /// /// /// /// /// /// /// ///

/// \remark РАБОТА С КОЛЬЦЕВЫМ БУФЕРОМ ДМА

///
/// \brief получить количество блоков ДМА, готовых для чтения
/// \param cfd дескриптор канала
/// \return кол-во блоков ДМА, готовых к чтению
///
MIL1553UDEXT_API unsigned int extmil1553ud_getCountDmaBlocksReadyRead(Mil1553ChannelFD cfd, unsigned int* codeError);


///
/// \brief mil1553ud_newDmaReadBlock создать контейнер блоков ДМА
/// выделяет под контейнер память
/// \param countDmaBlocks размер контейнера в блоках (штуках)
/// \return указатель на контейнер
///
MIL1553UDEXT_API DMA_READ_BLOCK* _ext_mil1553ud_newDmaReadBlock(unsigned int countDmaBlocks);

// для избежания коллизии имён с библиотекой драйвера, используйте макрос:
#ifndef mil1553ud_newDmaReadBlock
#define mil1553ud_newDmaReadBlock(countDmaBlocks) _ext_mil1553ud_newDmaReadBlock(countDmaBlocks)
#endif

///
/// \brief mil1553ud_freeDmaReadBlock освобождает контейнер блоков ДМА
/// \param block указатель на контейнер
///
MIL1553UDEXT_API void _ext_mil1553ud_freeDmaReadBlock(DMA_READ_BLOCK* block);

// для избежания коллизии имён с библиотекой драйвера, используйте макрос:
#ifndef mil1553ud_freeDmaReadBlock
#define mil1553ud_freeDmaReadBlock(block) _ext_mil1553ud_freeDmaReadBlock(block)
#endif

///
/// \brief extmil1553ud_readAllDmaBlocksReadyRead считывает все блоки ДМА, готовые к чтению, из канала
/// выделяет под контейнер память
/// \param cfd дескриптор канала
/// \return указатель на контейнер
///
MIL1553UDEXT_API DMA_READ_BLOCK* extmil1553ud_readAllDmaBlocksReadyRead(Mil1553ChannelFD cfd, unsigned int *cntBlocks);

/// \remark РАБОТА С КОЛЬЦЕВЫМ БУФЕРОМ ПРЕРЫВАНИЙ

///
/// \brief mil1553ud_getCountInterruptBlocksReadyRead получить количество блоков прерываний, готовых для чтения
/// \param cfd дескриптор канала
/// \return кол-во блоков, готовых к чтению
///
MIL1553UDEXT_API unsigned int extmil1553ud_getCountInterruptBlocksReadyRead(Mil1553ChannelFD cfd);

///
/// \brief mil1553ud_readInterruptBlocks считывает блоки прерываний, готовые к чтению, из канала
/// \param cfd дескриптор канала
/// \param block указатель на контейнер
/// \return результат операции
///
MIL1553UDEXT_API unsigned int extmil1553ud_readInterruptBlocks(Mil1553ChannelFD cfd, INTERRUPT_BLOCK_BUFFER* block);

/// \remark РЕЖИМ КШ

///
/// \brief mil1553ud_bcProgram управление микропрограммой в режиме КШ
/// старт/стоп BCSTRT
/// \param cfd дескриптор канала
/// \param action действие (BC_PROGRAM_START, BC_PROGRAM_STOP)
///
MIL1553UDEXT_API unsigned int extmil1553ud_bcProgram(Mil1553ChannelFD cfd, unsigned int action);

///
/// \brief mil1553ud_bcSetStartInstructionAddress установить стартовый адрес интсрукции микропрограммы КШ
/// \param cfd дескриптор канала
/// \param startNumber стартовый номер (адрес)
///
MIL1553UDEXT_API unsigned int extmil1553ud_bcSetStartInstructionAddress(Mil1553ChannelFD cfd, unsigned int startNumber);

/// \remark РЕЖИМ КШ - РАБОТА С МИКРОПРОГРАММОЙ КШ

///
/// \brief mil1553ud_newBcRamBlock создать контейнер для микропрограммы КШ
/// \param type тип - инструкции, операции, данные
/// MIL_BC_RAM_TYPE_INSTRUCTION, MIL_BC_RAM_TYPE_OPERATION, MIL_BC_RAM_TYPE_DATA
/// \param sizeInDwords размер контейнера в 32-разядных (двойных) словах
/// \param shiftInDwords смещение в памяти КШ в 32-разядных (двойных) словах
/// \return указатель на контейнер для микропрограммы КШ
///
MIL1553UDEXT_API BC_RAM_BLOCK* _ext_mil1553ud_newBcRamBlock(unsigned int type, unsigned int sizeInDwords, unsigned int shiftInDwords);

// для избежания коллизии имён с библиотекой драйвера, используйте макрос:
#ifndef mil1553ud_newBcRamBlock
#define mil1553ud_newBcRamBlock(type, sizeInDwords, shiftInDwords) _ext_mil1553ud_newBcRamBlock(type, sizeInDwords, shiftInDwords)
#endif

///
/// \brief mil1553ud_freeBcRamBlock освободить контейнер для микропрограммы КШ
/// \param block указатель на контейнер для микропрограммы КШ
///
MIL1553UDEXT_API void _ext_mil1553ud_freeBcRamBlock(BC_RAM_BLOCK* block);

// для избежания коллизии имён с библиотекой драйвера, используйте макрос:
#ifndef mil1553ud_freeBcRamBlock
#define mil1553ud_freeBcRamBlock(block) _ext_mil1553ud_freeBcRamBlock(block)
#endif

/// \remark РЕЖИМ КШ - РАБОТА С МИКРОПРОГРАММОЙ КШ - ИНСТРУКЦИИ, ОПЕРАЦИИ

/// \remark РАСШИФРОВКА БЛОКОВ ДМА

/// \remark РАСШИФРОВКА БЛОКОВ ДМА - СЕРВИСНЫЕ КОНСТАНТЫ

/// \remark РАСШИФРОВКА БЛОКОВ ДМА - ФУНКЦИИ

///
/// \brief mil1553ud_dmaBlock_getTypeTransaction получить тип транзакции из блока ДМА
/// тип транзакции соттветствует значениям 1-10 (см. ГОСТ 52070)
/// \param dmaBlock тип блока ДМА
/// \param dmaBlock указатель на блок ДМА
/// \return тип транзакции
///
MIL1553UDEXT_API unsigned int _ext_mil1553ud_dmaBlock_getTypeTransaction(unsigned int typeDmaBlock, unsigned char* dmaBlock);

// для избежания коллизии имён с библиотекой драйвера, используйте макрос:
#ifndef mil1553ud_dmaBlock_getTypeTransaction
#define mil1553ud_dmaBlock_getTypeTransaction(typeDmaBlock, dmaBlock) _ext_mil1553ud_dmaBlock_getTypeTransaction(typeDmaBlock, dmaBlock)
#endif

///
/// \brief mil1553ud_dmaBlock_isSuccessTransaction проверить успешность транзакции
/// \param typeDmaBlock тип блока ДМА
/// \param dmaBlock указатель на блок ДМА
/// \return признак успешности транзакции
///
MIL1553UDEXT_API unsigned int _ext_mil1553ud_dmaBlock_isSuccessTransaction(unsigned int typeDmaBlock, unsigned char* dmaBlock);

// для избежания коллизии имён с библиотекой драйвера, используйте макрос:
#ifndef mil1553ud_dmaBlock_isSuccessTransaction
#define mil1553ud_dmaBlock_isSuccessTransaction(typeDmaBlock, dmaBlock) _ext_mil1553ud_dmaBlock_isSuccessTransaction(typeDmaBlock, dmaBlock)
#endif

///
/// \brief mil1553ud_dmaBlock_getFreeTimer получить значение freetimer блока ДМА
/// \param typeDmaBlock тип блока ДМА
/// \param dmaBlock указатель на блок ДМА
/// \return значение freetimer
///
MIL1553UDEXT_API unsigned int _ext_mil1553ud_dmaBlock_getFreeTimer(unsigned int typeDmaBlock, unsigned char* dmaBlock);

// для избежания коллизии имён с библиотекой драйвера, используйте макрос:
#ifndef mil1553ud_dmaBlock_getFreeTimer
#define mil1553ud_dmaBlock_getFreeTimer(typeDmaBlock, dmaBlock) _ext_mil1553ud_dmaBlock_getFreeTimer(typeDmaBlock, dmaBlock)
#endif

///
/// \brief mil1553ud_dmaBlock_getCountServiceWords16 получить количество служебных 16-разрядных слов
/// (включая dword 1-5 и служебные слова МКИО)
/// \param typeDmaBlock тип блока ДМА
/// \param dmaBlock указатель на блок ДМА
/// \return количество служебных 16-разрядных слов
///
MIL1553UDEXT_API unsigned int _ext_mil1553ud_dmaBlock_getCountServiceWords16(unsigned int typeDmaBlock, unsigned char* dmaBlock);

// для избежания коллизии имён с библиотекой драйвера, используйте макрос:
#ifndef mil1553ud_dmaBlock_getCountServiceWords16
#define mil1553ud_dmaBlock_getCountServiceWords16(typeDmaBlock, dmaBlock) _ext_mil1553ud_dmaBlock_getCountServiceWords16(typeDmaBlock, dmaBlock)
#endif

///
/// \brief mil1553ud_dmaBlock_getCountDataWords16 получить количество 16-разрядных слов данных
/// \param typeDmaBlock тип блока ДМА
/// \param dmaBlock указатель на блок ДМА
/// \return количество 16-разрядных слов данных
///
MIL1553UDEXT_API unsigned int _ext_mil1553ud_dmaBlock_getCountDataWords16(unsigned int typeDmaBlock, unsigned char* dmaBlock);

// для избежания коллизии имён с библиотекой драйвера, используйте макрос:
#ifndef mil1553ud_dmaBlock_getCountDataWords16
#define mil1553ud_dmaBlock_getCountDataWords16(typeDmaBlock, dmaBlock) _ext_mil1553ud_dmaBlock_getCountDataWords16(typeDmaBlock, dmaBlock)
#endif

///
/// \brief mil1553ud_dmaBlock_getActiveBus получить активную шину
/// (по которой была транзакция)
/// \param typeDmaBlock тип блока ДМА
/// \param dmaBlock указатель на блок ДМА
/// \return шина А или Б
/// MIL1553_BUS_A или MIL1553_BUS_B
///
MIL1553UDEXT_API unsigned int _ext_mil1553ud_dmaBlock_getActiveBus(unsigned int typeDmaBlock, unsigned char* dmaBlock);

// для избежания коллизии имён с библиотекой драйвера, используйте макрос:
#ifndef mil1553ud_dmaBlock_getActiveBus
#define mil1553ud_dmaBlock_getActiveBus(typeDmaBlock, dmaBlock) _ext_mil1553ud_dmaBlock_getActiveBus(typeDmaBlock, dmaBlock)
#endif

///
/// \brief mil1553ud_dmaBlock_getPtrDataPart получить указатель на часть с данными блока ДМА
/// \param dmaBlock указатель на блок ДМА
/// \return указатель на часть с данными блока ДМА
///
MIL1553UDEXT_API unsigned char* _ext_mil1553ud_dmaBlock_getPtrDataPart(unsigned char* dmaBlock);

// для избежания коллизии имён с библиотекой драйвера, используйте макрос:
#ifndef mil1553ud_dmaBlock_getPtrDataPart
#define mil1553ud_dmaBlock_getPtrDataPart(dmaBlock) _ext_mil1553ud_dmaBlock_getPtrDataPart(dmaBlock)
#endif

///
/// \brief mil1553ud_dmaBlock_tdModeSubaddress получить поддарес из блока ДМА в режиме ОУ
/// \param dmaBlock указатель на блок ДМА
/// \return поддарес
///
MIL1553UDEXT_API unsigned int _ext_mil1553ud_dmaBlock_tdModeSubaddress(unsigned char* dmaBlock);

// для избежания коллизии имён с библиотекой драйвера, используйте макрос:
#ifndef mil1553ud_dmaBlock_tdModeSubaddress
#define mil1553ud_dmaBlock_tdModeSubaddress(dmaBlock) _ext_mil1553ud_dmaBlock_tdModeSubaddress(dmaBlock)
#endif

#ifdef __cplusplus
}
#endif

#endif // EXTMIL1553UD_LIBRARY_H
